@@ -105,7 +105,7 @@ static void handleItemClick(XEvent * event, void *data);
 
 static W_Class EditMenuItemClass = 0;
 
-W_Class InitEditMenuItem(WMScreen * scr)
+W_Class InitEditMenuItem(void)
 {
 	/* register our widget with WINGs and get our widget class ID */
 	if (!EditMenuItemClass) {
@@ -115,13 +115,13 @@ W_Class InitEditMenuItem(WMScreen * scr)
 	return EditMenuItemClass;
 }
 
-WEditMenuItem *WCreateEditMenuItem(WMWidget * parent, char *title, Bool isTitle)
+WEditMenuItem *WCreateEditMenuItem(WMWidget * parent, const char *title, Bool isTitle)
 {
 	WEditMenuItem *iPtr;
 	WMScreen *scr = WMWidgetScreen(parent);
 
 	if (!EditMenuItemClass)
-		InitEditMenuItem(scr);
+		InitEditMenuItem();
 
 	iPtr = wmalloc(sizeof(WEditMenuItem));
 
@@ -284,7 +284,7 @@ static void deselectItem(WEditMenu * menu);
 
 static W_Class EditMenuClass = 0;
 
-W_Class InitEditMenu(WMScreen * scr)
+W_Class InitEditMenu(void)
 {
 	/* register our widget with WINGs and get our widget class ID */
 	if (!EditMenuClass) {
@@ -334,6 +334,9 @@ static void realizeObserver(void *self, WMNotification * not)
 	WEditMenu *menu = (WEditMenu *) self;
 	GNUstepWMAttributes attribs;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) not;
+
 	memset(&attribs, 0, sizeof(GNUstepWMAttributes));
 	attribs.flags = GSWindowStyleAttr | GSWindowLevelAttr;
 	attribs.window_style = WMBorderlessWindowMask;
@@ -362,13 +365,13 @@ static void itemSelectObserver(void *self, WMNotification * notif)
 	}
 }
 
-static WEditMenu *makeEditMenu(WMScreen * scr, WMWidget * parent, char *title)
+static WEditMenu *makeEditMenu(WMScreen * scr, WMWidget * parent, const char *title)
 {
 	WEditMenu *mPtr;
 	WEditMenuItem *item;
 
 	if (!EditMenuClass)
-		InitEditMenu(scr);
+		InitEditMenu();
 
 	mPtr = wmalloc(sizeof(WEditMenu));
 
@@ -417,7 +420,7 @@ static WEditMenu *makeEditMenu(WMScreen * scr, WMWidget * parent, char *title)
 	return mPtr;
 }
 
-WEditMenu *WCreateEditMenu(WMScreen * scr, char *title)
+WEditMenu *WCreateEditMenu(WMScreen * scr, const char *title)
 {
 	return makeEditMenu(scr, NULL, title);
 }
@@ -432,7 +435,7 @@ void WSetEditMenuDelegate(WEditMenu * mPtr, WEditMenuDelegate * delegate)
 	mPtr->delegate = delegate;
 }
 
-WEditMenuItem *WInsertMenuItemWithTitle(WEditMenu * mPtr, int index, char *title)
+WEditMenuItem *WInsertMenuItemWithTitle(WEditMenu * mPtr, int index, const char *title)
 {
 	WEditMenuItem *item;
 
@@ -463,12 +466,12 @@ WEditMenuItem *WGetEditMenuItem(WEditMenu * mPtr, int index)
 	return WMGetFromArray(mPtr->items, index + (mPtr->flags.isTitled ? 1 : 0));
 }
 
-WEditMenuItem *WAddMenuItemWithTitle(WEditMenu * mPtr, char *title)
+WEditMenuItem *WAddMenuItemWithTitle(WEditMenu * mPtr, const char *title)
 {
 	return WInsertMenuItemWithTitle(mPtr, WMGetArrayItemCount(mPtr->items), title);
 }
 
-void WSetEditMenuTitle(WEditMenu * mPtr, char *title)
+void WSetEditMenuTitle(WEditMenu * mPtr, const char *title)
 {
 	WEditMenuItem *item;
 
@@ -504,7 +507,7 @@ void WSetEditMenuSubmenu(WEditMenu * mPtr, WEditMenuItem * item, WEditMenu * sub
 	paintEditMenuItem(item);
 }
 
-WEditMenu *WGetEditMenuSubmenu(WEditMenu * mPtr, WEditMenuItem * item)
+WEditMenu *WGetEditMenuSubmenu(WEditMenuItem *item)
 {
 	return item->submenu;
 }
@@ -571,6 +574,9 @@ static void closeMenuAction(WMWidget * w, void *data)
 {
 	WEditMenu *menu = (WEditMenu *) data;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) w;
+
 	WMAddIdleHandler(WMDestroyWidget, menu->closeB);
 	menu->closeB = NULL;
 
@@ -616,7 +622,7 @@ void WEditMenuHide(WEditMenu * mPtr)
 	while ((item = WGetEditMenuItem(mPtr, i++))) {
 		WEditMenu *submenu;
 
-		submenu = WGetEditMenuSubmenu(mPtr, item);
+		submenu = WGetEditMenuSubmenu(item);
 		if (submenu) {
 			WEditMenuHide(submenu);
 		}
@@ -634,7 +640,7 @@ void WEditMenuUnhide(WEditMenu * mPtr)
 	while ((item = WGetEditMenuItem(mPtr, i++))) {
 		WEditMenu *submenu;
 
-		submenu = WGetEditMenuSubmenu(mPtr, item);
+		submenu = WGetEditMenuSubmenu(item);
 		if (submenu) {
 			WEditMenuUnhide(submenu);
 		}
@@ -803,13 +809,9 @@ static void handleEvents(XEvent * event, void *data)
 
 static void stopEditItem(WEditMenu * menu, Bool apply)
 {
-	char *text;
-
 	if (apply) {
-		text = WMGetTextFieldText(menu->tfield);
-
 		wfree(menu->selectedItem->label);
-		menu->selectedItem->label = wstrdup(text);
+		menu->selectedItem->label = WMGetTextFieldText(menu->tfield);
 
 		updateMenuContents(menu);
 
@@ -938,6 +940,10 @@ static void slideWindow(Display * dpy, Window win, int srcX, int srcY, int dstX,
 
 static int errorHandler(Display * d, XErrorEvent * ev)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) d;
+	(void) ev;
+
 	/* just ignore */
 	return 0;
 }
@@ -991,7 +997,7 @@ static WEditMenu *findMenuInWindow(Display * dpy, Window toplevel, int x, int y)
 	return NULL;
 }
 
-static void handleDragOver(WEditMenu * menu, WMView * view, WEditMenuItem * item, int x, int y)
+static void handleDragOver(WEditMenu *menu, WMView *view, WEditMenuItem *item, int y)
 {
 	WMScreen *scr = W_VIEW_SCREEN(menu->view);
 	Window blaw;
@@ -1009,7 +1015,7 @@ static void handleDragOver(WEditMenu * menu, WMView * view, WEditMenuItem * item
 	}
 }
 
-static void handleItemDrop(WEditMenu * menu, WEditMenuItem * item, int x, int y)
+static void handleItemDrop(WEditMenu *menu, WEditMenuItem *item, int y)
 {
 	WMScreen *scr = W_VIEW_SCREEN(menu->view);
 	Window blaw;
@@ -1215,7 +1221,7 @@ static void dragItem(WEditMenu * menu, WEditMenuItem * item, Bool copy)
 			dmenu = findMenuInWindow(dpy, win, x, y);
 
 			if (dmenu) {
-				handleDragOver(dmenu, dview, dritem, x - dx, y - dy);
+				handleDragOver(dmenu, dview, dritem, y - dy);
 				if (!enteredMenu) {
 					enteredMenu = True;
 					XRecolorCursor(dpy, scr->defaultCursor, &green, &back);
@@ -1261,7 +1267,7 @@ static void dragItem(WEditMenu * menu, WEditMenuItem * item, Bool copy)
 
 			if (!menu->flags.isFactory && !copy) {
 				WRemoveEditMenuItem(menu, dritem);
-				handleItemDrop(dmenu ? dmenu : menu, dritem, orix, oriy);
+				handleItemDrop(dmenu ? dmenu : menu, dritem, oriy);
 			}
 		} else {
 			WRemoveEditMenuItem(menu, dritem);
@@ -1273,7 +1279,7 @@ static void dragItem(WEditMenu * menu, WEditMenuItem * item, Bool copy)
 			(*menu->delegate->itemCloned) (menu->delegate, menu, item, dritem);
 		}
 
-		handleItemDrop(dmenu, dritem, x - dy, y - dy);
+		handleItemDrop(dmenu, dritem, y - dy);
 
 		if (item->submenu && (menu->flags.isFactory || copy)) {
 			WEditMenu *submenu;
