@@ -4,6 +4,7 @@
  *
  *  Copyright (c) 1997-2003 Alfredo K. Kojima
  *  Copyright (c) 1998-2003 Dan Pascu
+ *  Copyright (c) 2014 Window Maker Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -34,6 +35,7 @@
 #include <sys/stat.h>
 #include <dirent.h>
 #include <limits.h>
+#include <errno.h>
 
 #ifdef HAVE_MALLOC_H
 #include <malloc.h>
@@ -51,37 +53,40 @@
 #include "WindowMaker.h"
 #include "GNUstep.h"
 #include "screen.h"
+#include "window.h"
 #include "dialog.h"
-#include "funcs.h"
+#include "misc.h"
 #include "stacking.h"
 #include "framewin.h"
 #include "window.h"
 #include "actions.h"
 #include "xinerama.h"
 
-extern WPreferences wPreferences;
 
 static WMPoint getCenter(WScreen * scr, int width, int height)
 {
 	return wGetPointToCenterRectInHead(scr, wGetHeadForPointerLocation(scr), width, height);
 }
 
-int wMessageDialog(WScreen * scr, char *title, char *message, char *defBtn, char *altBtn, char *othBtn)
+int wMessageDialog(WScreen *scr, const char *title, const char *message, const char *defBtn, const char *altBtn, const char *othBtn)
 {
 	WMAlertPanel *panel;
 	Window parent;
 	WWindow *wwin;
 	int result;
 	WMPoint center;
+	int pwidth, pheight;
 
-	panel = WMCreateAlertPanel(scr->wmscreen, NULL, title, message, defBtn, altBtn, othBtn);
+	panel = WMCreateScaledAlertPanel(scr->wmscreen, NULL, title, message, defBtn, altBtn, othBtn);
+	pwidth = WMWidgetWidth(panel->win);
+	pheight = WMWidgetHeight(panel->win);
 
-	parent = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, 400, 180, 0, 0, 0);
+	parent = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, pwidth, pheight, 0, 0, 0);
 
 	XReparentWindow(dpy, WMWidgetXID(panel->win), parent, 0, 0);
 
-	center = getCenter(scr, 400, 180);
-	wwin = wManageInternalWindow(scr, parent, None, NULL, center.x, center.y, 400, 180);
+	center = getCenter(scr, pwidth, pheight);
+	wwin = wManageInternalWindow(scr, parent, None, NULL, center.x, center.y, pwidth, pheight);
 	wwin->client_leader = WMWidgetXID(panel->win);
 
 	WMMapWidget(panel->win);
@@ -105,10 +110,13 @@ int wMessageDialog(WScreen * scr, char *title, char *message, char *defBtn, char
 
 static void toggleSaveSession(WMWidget *w, void *data)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) data;
+
 	wPreferences.save_session_on_exit = WMGetButtonSelected((WMButton *) w);
 }
 
-int wExitDialog(WScreen * scr, char *title, char *message, char *defBtn, char *altBtn, char *othBtn)
+int wExitDialog(WScreen *scr, const char *title, const char *message, const char *defBtn, const char *altBtn, const char *othBtn)
 {
 	WMAlertPanel *panel;
 	WMButton *saveSessionBtn;
@@ -116,24 +124,27 @@ int wExitDialog(WScreen * scr, char *title, char *message, char *defBtn, char *a
 	WWindow *wwin;
 	WMPoint center;
 	int result;
+	int pwidth, pheight;
 
-	panel = WMCreateAlertPanel(scr->wmscreen, NULL, title, message, defBtn, altBtn, othBtn);
+	panel = WMCreateScaledAlertPanel(scr->wmscreen, NULL, title, message, defBtn, altBtn, othBtn);
+	pwidth = WMWidgetWidth(panel->win);
+	pheight = WMWidgetHeight(panel->win);
 
 	/* add save session button */
 	saveSessionBtn = WMCreateSwitchButton(panel->hbox);
 	WMSetButtonAction(saveSessionBtn, toggleSaveSession, NULL);
-	WMAddBoxSubview(panel->hbox, WMWidgetView(saveSessionBtn), False, True, 200, 0, 0);
+	WMAddBoxSubview(panel->hbox, WMWidgetView(saveSessionBtn), False, True, pwidth / 2, 0, 0);
 	WMSetButtonText(saveSessionBtn, _("Save workspace state"));
 	WMSetButtonSelected(saveSessionBtn, wPreferences.save_session_on_exit);
 	WMRealizeWidget(saveSessionBtn);
 	WMMapWidget(saveSessionBtn);
 
-	parent = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, 400, 180, 0, 0, 0);
+	parent = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, pwidth, pheight, 0, 0, 0);
 
 	XReparentWindow(dpy, WMWidgetXID(panel->win), parent, 0, 0);
 
-	center = getCenter(scr, 400, 180);
-	wwin = wManageInternalWindow(scr, parent, None, NULL, center.x, center.y, 400, 180);
+	center = getCenter(scr, pwidth, pheight);
+	wwin = wManageInternalWindow(scr, parent, None, NULL, center.x, center.y, pwidth, pheight);
 
 	wwin->client_leader = WMWidgetXID(panel->win);
 
@@ -167,7 +178,7 @@ typedef struct _WMInputPanelWithHistory {
 	int varpos;
 } WMInputPanelWithHistory;
 
-static char *HistoryFileName(char *name)
+static char *HistoryFileName(const char *name)
 {
 	char *filename = NULL;
 
@@ -185,35 +196,50 @@ static int strmatch(const void *str1, const void *str2)
 	return !strcmp((const char *)str1, (const char *)str2);
 }
 
-static WMArray *LoadHistory(char *filename, int max)
+static WMArray *LoadHistory(const char *filename, int max)
 {
 	WMPropList *plhistory;
 	WMPropList *plitem;
 	WMArray *history;
 	int i, num;
+	char *str;
 
 	history = WMCreateArrayWithDestructor(1, wfree);
 	WMAddToArray(history, wstrdup(""));
 
-	plhistory = WMReadPropListFromFile((char *)filename);
+	plhistory = WMReadPropListFromFile(filename);
 
-	if (plhistory && WMIsPLArray(plhistory)) {
-		num = WMGetPropListItemCount(plhistory);
-		if (num > max)
-			num = max;
+	if (plhistory) {
+		if (WMIsPLArray(plhistory)) {
+			num = WMGetPropListItemCount(plhistory);
 
-		for (i = 0; i < num; ++i) {
-			plitem = WMGetFromPLArray(plhistory, i);
-			if (WMIsPLString(plitem) && WMFindInArray(history, strmatch,
-								  WMGetFromPLString(plitem)) == WANotFound)
-				WMAddToArray(history, WMGetFromPLString(plitem));
+			for (i = 0; i < num; ++i) {
+				plitem = WMGetFromPLArray(plhistory, i);
+				if (WMIsPLString(plitem)) {
+					str = WMGetFromPLString(plitem);
+					if (WMFindInArray(history, strmatch, str) == WANotFound) {
+						/*
+						 * The string here is duplicated because it will be freed
+						 * automatically when the array is deleted. This is not really
+						 * great because it is already an allocated string,
+						 * unfortunately we cannot re-use it because it will be freed
+						 * when we discard the PL (and we don't want to waste the PL's
+						 * memory either)
+						 */
+						WMAddToArray(history, wstrdup(str));
+						if (--max <= 0)
+							break;
+					}
+				}
+			}
 		}
+		WMReleasePropList(plhistory);
 	}
 
 	return history;
 }
 
-static void SaveHistory(WMArray * history, char *filename)
+static void SaveHistory(WMArray * history, const char *filename)
 {
 	int i;
 	WMPropList *plhistory;
@@ -223,7 +249,7 @@ static void SaveHistory(WMArray * history, char *filename)
 	for (i = 0; i < WMGetArrayItemCount(history); ++i)
 		WMAddToPLArray(plhistory, WMCreatePLString(WMGetFromArray(history, i)));
 
-	WMWritePropListToFile(plhistory, (char *)filename);
+	WMWritePropListToFile(plhistory, filename);
 	WMReleasePropList(plhistory);
 }
 
@@ -242,7 +268,8 @@ ScanFiles(const char *dir, const char *prefix, unsigned acceptmask, unsigned dec
 	char *fullfilename, *suffix;
 
 	prefixlen = strlen(prefix);
-	if ((d = opendir(dir)) != NULL) {
+	d = opendir(dir);
+	if (d != NULL) {
 		while ((de = readdir(d)) != NULL) {
 			if (strlen(de->d_name) > prefixlen &&
 			    !strncmp(prefix, de->d_name, prefixlen) &&
@@ -257,7 +284,7 @@ ScanFiles(const char *dir, const char *prefix, unsigned acceptmask, unsigned dec
 						  de->d_name + prefixlen) == WANotFound) {
 					suffix = wstrdup(de->d_name + prefixlen);
 					if (sb.st_mode & S_IFDIR)
-						wstrappend(suffix,"/");
+						suffix = wstrappend(suffix, "/");
 					WMAddToArray(result, suffix);
 				}
 				wfree(fullfilename);
@@ -278,12 +305,14 @@ static WMArray *GenerateVariants(const char *complete)
 	while (*complete == ' ')
 		++complete;
 
-	if ((pos = strrchr(complete, ' ')) != NULL) {
+	pos = strrchr(complete, ' ');
+	if (pos != NULL) {
 		complete = pos + 1;
 		firstWord = False;
 	}
 
-	if ((pos = strrchr(complete, '/')) != NULL) {
+	pos = strrchr(complete, '/');
+	if (pos != NULL) {
 		tmp = wstrndup((char *)complete, pos - complete + 1);
 		if (*tmp == '~' && *(tmp + 1) == '/' && getenv("HOME")) {
 			dir = wstrdup(getenv("HOME"));
@@ -391,7 +420,7 @@ static void handleHistoryKeyPress(XEvent * event, void *clientData)
 	}
 }
 
-int wAdvancedInputDialog(WScreen * scr, char *title, char *message, char *name, char **text)
+int wAdvancedInputDialog(WScreen *scr, const char *title, const char *message, const char *name, char **text)
 {
 	WWindow *wwin;
 	Window parent;
@@ -399,10 +428,11 @@ int wAdvancedInputDialog(WScreen * scr, char *title, char *message, char *name,
 	WMPoint center;
 	WMInputPanelWithHistory *p;
 	char *filename;
+	int pwidth, pheight;
 
 	filename = HistoryFileName(name);
 	p = wmalloc(sizeof(WMInputPanelWithHistory));
-	p->panel = WMCreateInputPanel(scr->wmscreen, NULL, title, message, *text, _("OK"), _("Cancel"));
+	p->panel = WMCreateScaledInputPanel(scr->wmscreen, NULL, title, message, *text, _("OK"), _("Cancel"));
 	p->history = LoadHistory(filename, wPreferences.history_lines);
 	p->histpos = 0;
 	p->prefix = NULL;
@@ -411,14 +441,16 @@ int wAdvancedInputDialog(WScreen * scr, char *title, char *message, char *name,
 	p->variants = NULL;
 	p->varpos = 0;
 	WMCreateEventHandler(WMWidgetView(p->panel->text), KeyPressMask, handleHistoryKeyPress, p);
+	pwidth = WMWidgetWidth(p->panel->win);
+	pheight = WMWidgetHeight(p->panel->win);
 
-	parent = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, 320, 160, 0, 0, 0);
+	parent = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, pwidth, pheight, 0, 0, 0);
 	XSelectInput(dpy, parent, KeyPressMask | KeyReleaseMask);
 
 	XReparentWindow(dpy, WMWidgetXID(p->panel->win), parent, 0, 0);
 
-	center = getCenter(scr, 320, 160);
-	wwin = wManageInternalWindow(scr, parent, None, NULL, center.x, center.y, 320, 160);
+	center = getCenter(scr, pwidth, pheight);
+	wwin = wManageInternalWindow(scr, parent, None, NULL, center.x, center.y, pwidth, pheight);
 
 	wwin->client_leader = WMWidgetXID(p->panel->win);
 
@@ -455,23 +487,26 @@ int wAdvancedInputDialog(WScreen * scr, char *title, char *message, char *name,
 	}
 }
 
-int wInputDialog(WScreen * scr, char *title, char *message, char **text)
+int wInputDialog(WScreen *scr, const char *title, const char *message, char **text)
 {
 	WWindow *wwin;
 	Window parent;
 	WMInputPanel *panel;
 	char *result;
 	WMPoint center;
+	int pwidth, pheight;
 
-	panel = WMCreateInputPanel(scr->wmscreen, NULL, title, message, *text, _("OK"), _("Cancel"));
+	panel = WMCreateScaledInputPanel(scr->wmscreen, NULL, title, message, *text, _("OK"), _("Cancel"));
+	pwidth = WMWidgetWidth(panel->win);
+	pheight = WMWidgetHeight(panel->win);
 
-	parent = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, 320, 160, 0, 0, 0);
+	parent = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, pwidth, pheight, 0, 0, 0);
 	XSelectInput(dpy, parent, KeyPressMask | KeyReleaseMask);
 
 	XReparentWindow(dpy, WMWidgetXID(panel->win), parent, 0, 0);
 
-	center = getCenter(scr, 320, 160);
-	wwin = wManageInternalWindow(scr, parent, None, NULL, center.x, center.y, 320, 160);
+	center = getCenter(scr, pwidth, pheight);
+	wwin = wManageInternalWindow(scr, parent, None, NULL, center.x, center.y, pwidth, pheight);
 
 	wwin->client_leader = WMWidgetXID(panel->win);
 
@@ -539,7 +574,7 @@ typedef struct IconPanel {
 	short preview;
 } IconPanel;
 
-static void listPixmaps(WScreen * scr, WMList * lPtr, char *path)
+static void listPixmaps(WScreen *scr, WMList *lPtr, const char *path)
 {
 	struct dirent *dentry;
 	DIR *dir;
@@ -553,16 +588,11 @@ static void listPixmaps(WScreen * scr, WMList * lPtr, char *path)
 	dir = opendir(apath);
 
 	if (!dir) {
-		char *msg;
-		char *tmp;
-		tmp = _("Could not open directory ");
-		msg = wmalloc(strlen(tmp) + strlen(path) + 6);
-		strcpy(msg, tmp);
-		strcat(msg, path);
-
-		wMessageDialog(scr, _("Error"), msg, _("OK"), NULL, NULL);
-		wfree(msg);
 		wfree(apath);
+		snprintf(pbuf, sizeof(pbuf),
+		         _("Could not open directory \"%s\":\n%s"),
+		         path, strerror(errno));
+		wMessageDialog(scr, _("Error"), pbuf, _("OK"), NULL, NULL);
 		return;
 	}
 
@@ -573,9 +603,13 @@ static void listPixmaps(WScreen * scr, WMList * lPtr, char *path)
 		if (strcmp(dentry->d_name, ".") == 0 || strcmp(dentry->d_name, "..") == 0)
 			continue;
 
-		strcpy(pbuf, apath);
-		strcat(pbuf, "/");
-		strcat(pbuf, dentry->d_name);
+		if (wstrlcpy(pbuf, apath, sizeof(pbuf)) >= sizeof(pbuf) ||
+		    wstrlcat(pbuf, "/", sizeof(pbuf)) >= sizeof(pbuf) ||
+		    wstrlcat(pbuf, dentry->d_name, sizeof(pbuf)) >= sizeof(pbuf)) {
+			wwarning(_("full path for file \"%s\" in \"%s\" is longer than %d bytes, skipped"),
+			         dentry->d_name, path, (int) (sizeof(pbuf) - 1) );
+			continue;
+		}
 
 		if (stat(pbuf, &statb) < 0)
 			continue;
@@ -592,16 +626,20 @@ static void listPixmaps(WScreen * scr, WMList * lPtr, char *path)
 	panel->preview = True;
 }
 
-static void setViewedImage(IconPanel * panel, char *file)
+static void setViewedImage(IconPanel *panel, const char *file)
 {
 	WMPixmap *pixmap;
 	RColor color;
+	int iwidth, iheight;
 
 	color.red = 0xae;
 	color.green = 0xaa;
 	color.blue = 0xae;
 	color.alpha = 0;
-	pixmap = WMCreateBlendedPixmapFromFile(WMWidgetScreen(panel->win), file, &color);
+	iwidth = WMWidgetWidth(panel->iconView);
+	iheight = WMWidgetHeight(panel->iconView);
+	pixmap = WMCreateScaledBlendedPixmapFromFile(WMWidgetScreen(panel->win), file, &color, iwidth, iheight);
+
 	if (!pixmap) {
 		WMSetButtonEnabled(panel->okButton, False);
 
@@ -645,13 +683,13 @@ static void listCallback(void *self, void *data)
 		if (item == NULL)
 			return;
 		path = item->text;
-		tmp = wexpandpath(path);
 
 		item = WMGetListSelectedItem(panel->iconList);
 		if (item == NULL)
 			return;
 		iconFile = item->text;
 
+		tmp = wexpandpath(path);
 		path = wmalloc(strlen(tmp) + strlen(iconFile) + 4);
 		strcpy(path, tmp);
 		strcat(path, "/");
@@ -700,6 +738,9 @@ static void drawIconProc(WMList * lPtr, int index, Drawable d, char *text, int s
 	RColor color;
 	int x, y, width, height, len;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) index;
+
 	if (!panel->preview)
 		return;
 
@@ -721,7 +762,7 @@ static void drawIconProc(WMList * lPtr, int index, Drawable d, char *text, int s
 	color.blue = WMBlueComponentOfColor(back) >> 8;
 	color.alpha = WMGetColorAlpha(back) >> 8;
 
-	pixmap = WMCreateBlendedPixmapFromFile(wmscr, file, &color);
+	pixmap = WMCreateScaledBlendedPixmapFromFile(wmscr, file, &color, width - 2, height - 2);
 	wfree(file);
 
 	if (!pixmap) {
@@ -875,7 +916,7 @@ static void keyPressHandler(XEvent * event, void *data)
 	}
 }
 
-Bool wIconChooserDialog(WScreen * scr, char **file, char *instance, char *class)
+Bool wIconChooserDialog(WScreen *scr, char **file, const char *instance, const char *class)
 {
 	WWindow *wwin;
 	Window parent;
@@ -883,22 +924,27 @@ Bool wIconChooserDialog(WScreen * scr, char **file, char *instance, char *class)
 	WMColor *color;
 	WMFont *boldFont;
 	Bool result;
+	int wmScaleWidth, wmScaleHeight;
+	int pwidth, pheight;
 
 	panel = wmalloc(sizeof(IconPanel));
 
 	panel->scr = scr;
 
 	panel->win = WMCreateWindow(scr->wmscreen, "iconChooser");
-	WMResizeWidget(panel->win, 450, 280);
+	WMGetScaleBaseFromSystemFont(scr->wmscreen, &wmScaleWidth, &wmScaleHeight);
+	pwidth = WMScaleX(450);
+	pheight = WMScaleY(280);
+	WMResizeWidget(panel->win, pwidth, pheight);
 
 	WMCreateEventHandler(WMWidgetView(panel->win), KeyPressMask | KeyReleaseMask, keyPressHandler, panel);
 
-	boldFont = WMBoldSystemFontOfSize(scr->wmscreen, 12);
-	panel->normalfont = WMSystemFontOfSize(WMWidgetScreen(panel->win), 12);
+	boldFont = WMBoldSystemFontOfSize(scr->wmscreen, WMScaleY(12));
+	panel->normalfont = WMSystemFontOfSize(WMWidgetScreen(panel->win), WMScaleY(12));
 
 	panel->dirLabel = WMCreateLabel(panel->win);
-	WMResizeWidget(panel->dirLabel, 200, 20);
-	WMMoveWidget(panel->dirLabel, 10, 7);
+	WMResizeWidget(panel->dirLabel, WMScaleX(200), WMScaleY(20));
+	WMMoveWidget(panel->dirLabel, WMScaleX(10), WMScaleY(7));
 	WMSetLabelText(panel->dirLabel, _("Directories"));
 	WMSetLabelFont(panel->dirLabel, boldFont);
 	WMSetLabelTextAlignment(panel->dirLabel, WACenter);
@@ -906,8 +952,8 @@ Bool wIconChooserDialog(WScreen * scr, char **file, char *instance, char *class)
 	WMSetLabelRelief(panel->dirLabel, WRSunken);
 
 	panel->iconLabel = WMCreateLabel(panel->win);
-	WMResizeWidget(panel->iconLabel, 140, 20);
-	WMMoveWidget(panel->iconLabel, 215, 7);
+	WMResizeWidget(panel->iconLabel, WMScaleX(140), WMScaleY(20));
+	WMMoveWidget(panel->iconLabel, WMScaleX(215), WMScaleY(7));
 	WMSetLabelText(panel->iconLabel, _("Icons"));
 	WMSetLabelFont(panel->iconLabel, boldFont);
 	WMSetLabelTextAlignment(panel->iconLabel, WACenter);
@@ -927,84 +973,102 @@ Bool wIconChooserDialog(WScreen * scr, char **file, char *instance, char *class)
 	WMSetLabelRelief(panel->iconLabel, WRSunken);
 
 	panel->dirList = WMCreateList(panel->win);
-	WMResizeWidget(panel->dirList, 200, 170);
-	WMMoveWidget(panel->dirList, 10, 30);
+	WMResizeWidget(panel->dirList, WMScaleX(200), WMScaleY(170));
+	WMMoveWidget(panel->dirList, WMScaleX(10), WMScaleY(30));
 	WMSetListAction(panel->dirList, listCallback, panel);
 
 	panel->iconList = WMCreateList(panel->win);
-	WMResizeWidget(panel->iconList, 140, 170);
-	WMMoveWidget(panel->iconList, 215, 30);
+	WMResizeWidget(panel->iconList, WMScaleX(140), WMScaleY(170));
+	WMMoveWidget(panel->iconList, WMScaleX(215), WMScaleY(30));
 	WMSetListAction(panel->iconList, listCallback, panel);
 
 	WMHangData(panel->iconList, panel);
 
 	panel->previewButton = WMCreateCommandButton(panel->win);
-	WMResizeWidget(panel->previewButton, 75, 26);
-	WMMoveWidget(panel->previewButton, 365, 130);
+	WMResizeWidget(panel->previewButton, WMScaleX(75), WMScaleY(26));
+	WMMoveWidget(panel->previewButton, WMScaleX(365), WMScaleY(130));
 	WMSetButtonText(panel->previewButton, _("Preview"));
 	WMSetButtonAction(panel->previewButton, buttonCallback, panel);
 
 	panel->iconView = WMCreateLabel(panel->win);
-	WMResizeWidget(panel->iconView, 75, 75);
-	WMMoveWidget(panel->iconView, 365, 40);
+	WMResizeWidget(panel->iconView, WMScaleX(75), WMScaleY(75));
+	WMMoveWidget(panel->iconView, WMScaleX(365), WMScaleY(40));
 	WMSetLabelImagePosition(panel->iconView, WIPOverlaps);
 	WMSetLabelRelief(panel->iconView, WRSunken);
 	WMSetLabelTextAlignment(panel->iconView, WACenter);
 
 	panel->fileLabel = WMCreateLabel(panel->win);
-	WMResizeWidget(panel->fileLabel, 80, 20);
-	WMMoveWidget(panel->fileLabel, 10, 210);
+	WMResizeWidget(panel->fileLabel, WMScaleX(80), WMScaleY(20));
+	WMMoveWidget(panel->fileLabel, WMScaleX(10), WMScaleY(210));
 	WMSetLabelText(panel->fileLabel, _("File Name:"));
 
 	panel->fileField = WMCreateTextField(panel->win);
 	WMSetViewNextResponder(WMWidgetView(panel->fileField), WMWidgetView(panel->win));
-	WMResizeWidget(panel->fileField, 345, 20);
-	WMMoveWidget(panel->fileField, 95, 210);
+	WMResizeWidget(panel->fileField, WMScaleX(345), WMScaleY(20));
+	WMMoveWidget(panel->fileField, WMScaleX(95), WMScaleY(210));
 	WMSetTextFieldEditable(panel->fileField, False);
 
 	panel->okButton = WMCreateCommandButton(panel->win);
-	WMResizeWidget(panel->okButton, 80, 26);
-	WMMoveWidget(panel->okButton, 360, 240);
+	WMResizeWidget(panel->okButton, WMScaleX(80), WMScaleY(26));
+	WMMoveWidget(panel->okButton, WMScaleX(360), WMScaleY(242));
 	WMSetButtonText(panel->okButton, _("OK"));
 	WMSetButtonEnabled(panel->okButton, False);
 	WMSetButtonAction(panel->okButton, buttonCallback, panel);
 
 	panel->cancelButton = WMCreateCommandButton(panel->win);
-	WMResizeWidget(panel->cancelButton, 80, 26);
-	WMMoveWidget(panel->cancelButton, 270, 240);
+	WMResizeWidget(panel->cancelButton, WMScaleX(80), WMScaleY(26));
+	WMMoveWidget(panel->cancelButton, WMScaleX(270), WMScaleY(242));
 	WMSetButtonText(panel->cancelButton, _("Cancel"));
 	WMSetButtonAction(panel->cancelButton, buttonCallback, panel);
 #if 0
 	panel->chooseButton = WMCreateCommandButton(panel->win);
-	WMResizeWidget(panel->chooseButton, 110, 26);
-	WMMoveWidget(panel->chooseButton, 150, 240);
+	WMResizeWidget(panel->chooseButton, WMScaleX(110), WMScaleY(26));
+	WMMoveWidget(panel->chooseButton, WMScaleX(150), WMScaleY(242));
 	WMSetButtonText(panel->chooseButton, _("Choose File"));
 	WMSetButtonAction(panel->chooseButton, buttonCallback, panel);
 #endif
 	WMRealizeWidget(panel->win);
 	WMMapSubwidgets(panel->win);
 
-	parent = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, 450, 280, 0, 0, 0);
+	parent = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, pwidth, pheight, 0, 0, 0);
 
 	XReparentWindow(dpy, WMWidgetXID(panel->win), parent, 0, 0);
 
 	{
-		char *tmp;
-		int len = (instance ? strlen(instance) : 0)
-		    + (class ? strlen(class) : 0) + 32;
+		static const char *prefix = NULL;
+		char *title;
+		int len;
 		WMPoint center;
 
-		tmp = wmalloc(len);
+		if (prefix == NULL)
+			prefix = _("Icon Chooser");
 
-		if (tmp && (instance || class))
-			snprintf(tmp, len, "%s [%s.%s]", _("Icon Chooser"), instance, class);
-		else
-			strcpy(tmp, _("Icon Chooser"));
+		len = strlen(prefix)
+			+ 2	// " ["
+			+ (instance ? strlen(instance) : 0)
+			+ 1	// "."
+			+ (class ? strlen(class) : 0)
+			+ 1	// "]"
+			+ 1;	// final NUL
 
-		center = getCenter(scr, 450, 280);
+		title = wmalloc(len);
+		strcpy(title, prefix);
 
-		wwin = wManageInternalWindow(scr, parent, None, tmp, center.x, center.y, 450, 280);
-		wfree(tmp);
+		if (instance || class) {
+			strcat(title, " [");
+			if (instance != NULL)
+				strcat(title, instance);
+			if (instance && class)
+				strcat(title, ".");
+			if (class != NULL)
+				strcat(title, class);
+			strcat(title, "]");
+		}
+
+		center = getCenter(scr, pwidth, pheight);
+
+		wwin = wManageInternalWindow(scr, parent, None, title, center.x, center.y, pwidth, pheight);
+		wfree(title);
 	}
 
 	/* put icon paths in the list */
@@ -1084,29 +1148,36 @@ typedef struct {
 
 #define COPYRIGHT_TEXT  \
     "Copyright \xc2\xa9 1997-2006 Alfredo K. Kojima\n"\
-    "Copyright \xc2\xa9 1998-2006 Dan Pascu"
+    "Copyright \xc2\xa9 1998-2006 Dan Pascu\n"\
+    "Copyright \xc2\xa9 2013-2020 Window Maker Developers Team"
 
-static InfoPanel *thePanel = NULL;
+static InfoPanel *infoPanel = NULL;
 
 static void destroyInfoPanel(WCoreWindow *foo, void *data, XEvent *event)
 {
-	WMUnmapWidget(thePanel);
-	wUnmanageWindow(thePanel->wwin, False, False);
-	WMDestroyWidget(thePanel->win);
-	wfree(thePanel);
-	thePanel = NULL;
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) foo;
+	(void) data;
+	(void) event;
+
+	WMUnmapWidget(infoPanel);
+	wUnmanageWindow(infoPanel->wwin, False, False);
+	WMDestroyWidget(infoPanel->win);
+	wfree(infoPanel);
+	infoPanel = NULL;
 }
 
-void wShowInfoPanel(WScreen * scr)
+void wShowInfoPanel(WScreen *scr)
 {
 	InfoPanel *panel;
 	WMPixmap *logo;
 	WMFont *font;
 	char *strbuf = NULL;
+	const char *separator;
 	char buffer[256];
-	char *name;
 	Window parent;
 	WWindow *wwin;
+	WMPoint center;
 	char **strl;
 	int i, width = 50, sepHeight;
 	char *visuals[] = {
@@ -1117,11 +1188,13 @@ void wShowInfoPanel(WScreen * scr)
 		"TrueColor",
 		"DirectColor"
 	};
+	int wmScaleWidth, wmScaleHeight;
+	int pwidth, pheight;
 
-	if (thePanel) {
-		if (thePanel->scr == scr) {
-			wRaiseFrame(thePanel->wwin->frame->core);
-			wSetFocusTo(scr, thePanel->wwin);
+	if (infoPanel) {
+		if (infoPanel->scr == scr) {
+			wRaiseFrame(infoPanel->wwin->frame->core);
+			wSetFocusTo(scr, infoPanel->wwin);
 		}
 		return;
 	}
@@ -1131,7 +1204,10 @@ void wShowInfoPanel(WScreen * scr)
 	panel->scr = scr;
 
 	panel->win = WMCreateWindow(scr->wmscreen, "info");
-	WMResizeWidget(panel->win, 390, 230);
+	WMGetScaleBaseFromSystemFont(scr->wmscreen, &wmScaleWidth, &wmScaleHeight);
+	pwidth = WMScaleX(382);
+	pheight = WMScaleY(250);
+	WMResizeWidget(panel->win, pwidth, pheight);
 
 	logo = WMCreateApplicationIconBlendedPixmap(scr->wmscreen, (RColor *) NULL);
 	if (!logo) {
@@ -1139,20 +1215,23 @@ void wShowInfoPanel(WScreen * scr)
 	}
 	if (logo) {
 		panel->logoL = WMCreateLabel(panel->win);
-		WMResizeWidget(panel->logoL, 64, 64);
-		WMMoveWidget(panel->logoL, 30, 20);
+		WMResizeWidget(panel->logoL, WMScaleX(64), WMScaleY(64));
+		WMMoveWidget(panel->logoL, WMScaleX(30), WMScaleY(20));
 		WMSetLabelImagePosition(panel->logoL, WIPImageOnly);
 		WMSetLabelImage(panel->logoL, logo);
 		WMReleasePixmap(logo);
 	}
 
-	sepHeight = 3;
+	sepHeight = WMScaleY(3);
 	panel->name1L = WMCreateLabel(panel->win);
-	WMResizeWidget(panel->name1L, 240, 30 + 2);
-	WMMoveWidget(panel->name1L, 100, 30 - 2 - sepHeight);
+	WMResizeWidget(panel->name1L, WMScaleX(240), WMScaleY(30) + WMScaleY(2));
+	WMMoveWidget(panel->name1L, WMScaleX(100), WMScaleY(30) - WMScaleY(2) - sepHeight);
 
-	name = "Lucida Sans,Comic Sans MS,URW Gothic L,Trebuchet MS" ":italic:pixelsize=28:antialias=true";
-	font = WMCreateFont(scr->wmscreen, name);
+	snprintf(buffer,
+		sizeof(buffer),
+		"Lucida Sans,Comic Sans MS,URW Gothic L,Trebuchet MS:italic:pixelsize=%d:antialias=true",
+		WMScaleY(24));
+	font = WMCreateFont(scr->wmscreen, buffer);
 	strbuf = "Window Maker";
 	if (font) {
 		width = WMWidthOfString(font, strbuf, strlen(strbuf));
@@ -1164,15 +1243,15 @@ void wShowInfoPanel(WScreen * scr)
 
 	panel->lineF = WMCreateFrame(panel->win);
 	WMResizeWidget(panel->lineF, width, sepHeight);
-	WMMoveWidget(panel->lineF, 100 + (240 - width) / 2, 60 - sepHeight);
+	WMMoveWidget(panel->lineF, WMScaleX(100) + (WMScaleX(240) - width) / 2, WMScaleY(60) - sepHeight);
 	WMSetFrameRelief(panel->lineF, WRSimple);
 	WMSetWidgetBackgroundColor(panel->lineF, scr->black);
 
 	panel->name2L = WMCreateLabel(panel->win);
-	WMResizeWidget(panel->name2L, 240, 24);
-	WMMoveWidget(panel->name2L, 100, 60);
-	name = "URW Gothic L,Nimbus Sans L:pixelsize=16:antialias=true";
-	font = WMCreateFont(scr->wmscreen, name);
+	WMResizeWidget(panel->name2L, WMScaleX(240), WMScaleY(24));
+	WMMoveWidget(panel->name2L, WMScaleX(100), WMScaleY(60));
+	snprintf(buffer, sizeof(buffer), "URW Gothic L,Nimbus Sans L:pixelsize=%d:antialias=true", WMScaleY(16));
+	font = WMCreateFont(scr->wmscreen, buffer);
 	if (font) {
 		WMSetLabelFont(panel->name2L, font);
 		WMReleaseFont(font);
@@ -1183,18 +1262,18 @@ void wShowInfoPanel(WScreen * scr)
 
 	snprintf(buffer, sizeof(buffer), _("Version %s"), VERSION);
 	panel->versionL = WMCreateLabel(panel->win);
-	WMResizeWidget(panel->versionL, 310, 16);
-	WMMoveWidget(panel->versionL, 30, 95);
+	WMResizeWidget(panel->versionL, WMScaleX(310), WMScaleY(16));
+	WMMoveWidget(panel->versionL, WMScaleX(30), WMScaleY(95));
 	WMSetLabelTextAlignment(panel->versionL, WARight);
 	WMSetLabelText(panel->versionL, buffer);
 	WMSetLabelWraps(panel->versionL, False);
 
 	panel->copyrL = WMCreateLabel(panel->win);
-	WMResizeWidget(panel->copyrL, 360, 40);
-	WMMoveWidget(panel->copyrL, 15, 185);
+	WMResizeWidget(panel->copyrL, WMScaleX(360), WMScaleY(60));
+	WMMoveWidget(panel->copyrL, WMScaleX(15), WMScaleY(190));
 	WMSetLabelTextAlignment(panel->copyrL, WALeft);
 	WMSetLabelText(panel->copyrL, COPYRIGHT_TEXT);
-	font = WMSystemFontOfSize(scr->wmscreen, 11);
+	font = WMSystemFontOfSize(scr->wmscreen, WMScaleY(11));
 	if (font) {
 		WMSetLabelFont(panel->copyrL, font);
 		WMReleaseFont(font);
@@ -1228,26 +1307,53 @@ void wShowInfoPanel(WScreen * scr)
 	{
 		struct mallinfo ma = mallinfo();
 		snprintf(buffer, sizeof(buffer),
-			 _("Total memory allocated: %i kB (in use: %i kB).\n"),
-			 (ma.arena + ma.hblkhd) / 1024, (ma.uordblks + ma.hblkhd) / 1024);
+#ifdef DEBUG
+					_("Total memory allocated: %i kB (in use: %i kB, %d free chunks).\n"),
+#else
+					_("Total memory allocated: %i kB (in use: %i kB).\n"),
+#endif
+					(ma.arena + ma.hblkhd) / 1024,
+					(ma.uordblks + ma.hblkhd) / 1024
+#ifdef DEBUG
+					/*
+					 * This information is representative of the memory
+					 * fragmentation. In ideal case it should be 1, but
+					 * that is never possible
+					 */
+					, ma.ordblks
+#endif
+					);
 
 		strbuf = wstrappend(strbuf, buffer);
 	}
 #endif
 
-	strbuf = wstrappend(strbuf, _("Supported image formats: "));
+	strbuf = wstrappend(strbuf, _("Image formats: "));
 	strl = RSupportedFileFormats();
+	separator = NULL;
 	for (i = 0; strl[i] != NULL; i++) {
+		if (separator != NULL)
+			strbuf = wstrappend(strbuf, separator);
 		strbuf = wstrappend(strbuf, strl[i]);
-		strbuf = wstrappend(strbuf, " ");
+		separator = ", ";
 	}
 
-	strbuf = wstrappend(strbuf, _("\nAdditional support for: WMSPEC"));
-#ifdef MWM_HINTS
-	strbuf = wstrappend(strbuf, " and MWM");
+	strbuf = wstrappend(strbuf, _("\nAdditional support for: "));
+	strbuf = wstrappend(strbuf, "WMSPEC");
+
+#ifdef USE_MWM_HINTS
+	strbuf = wstrappend(strbuf, ", MWM");
 #endif
 
-#ifdef XINERAMA
+#ifdef USE_DOCK_XDND
+	strbuf = wstrappend(strbuf, ", XDnD");
+#endif
+
+#ifdef USE_MAGICK
+	strbuf = wstrappend(strbuf, ", ImageMagick");
+#endif
+
+#ifdef USE_XINERAMA
 	strbuf = wstrappend(strbuf, _("\n"));
 #ifdef SOLARIS_XINERAMA
 	strbuf = wstrappend(strbuf, _("Solaris "));
@@ -1255,16 +1361,26 @@ void wShowInfoPanel(WScreen * scr)
 	strbuf = wstrappend(strbuf, _("Xinerama: "));
 	{
 		char tmp[128];
-		snprintf(tmp, sizeof(tmp) - 1, "%d heads found.", scr->xine_info.count);
+		snprintf(tmp, sizeof(tmp) - 1, _("%d head(s) found."), scr->xine_info.count);
 		strbuf = wstrappend(strbuf, tmp);
 	}
 #endif
 
+#ifdef USE_RANDR
+	strbuf = wstrappend(strbuf, _("\n"));
+	strbuf = wstrappend(strbuf, "RandR: ");
+	if (w_global.xext.randr.supported)
+		strbuf = wstrappend(strbuf, _("supported"));
+	else
+		strbuf = wstrappend(strbuf, _("unsupported"));
+	strbuf = wstrappend(strbuf, ".");
+#endif
+
 	panel->infoL = WMCreateLabel(panel->win);
-	WMResizeWidget(panel->infoL, 350, 75);
-	WMMoveWidget(panel->infoL, 15, 115);
+	WMResizeWidget(panel->infoL, WMScaleX(350), WMScaleY(80));
+	WMMoveWidget(panel->infoL, WMScaleX(15), WMScaleY(115));
 	WMSetLabelText(panel->infoL, strbuf);
-	font = WMSystemFontOfSize(scr->wmscreen, 11);
+	font = WMSystemFontOfSize(scr->wmscreen, WMScaleY(11));
 	if (font) {
 		WMSetLabelFont(panel->infoL, font);
 		WMReleaseFont(font);
@@ -1275,17 +1391,14 @@ void wShowInfoPanel(WScreen * scr)
 	WMRealizeWidget(panel->win);
 	WMMapSubwidgets(panel->win);
 
-	parent = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, 382, 230, 0, 0, 0);
+	parent = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, pwidth, pheight, 0, 0, 0);
 
 	XReparentWindow(dpy, WMWidgetXID(panel->win), parent, 0, 0);
 
 	WMMapWidget(panel->win);
 
-	{
-		WMPoint center = getCenter(scr, 382, 230);
-
-		wwin = wManageInternalWindow(scr, parent, None, _("Info"), center.x, center.y, 382, 230);
-	}
+	center = getCenter(scr, pwidth, pheight);
+	wwin = wManageInternalWindow(scr, parent, None, _("Info"), center.x, center.y, pwidth, pheight);
 
 	WSETUFLAG(wwin, no_closable, 0);
 	WSETUFLAG(wwin, no_close_button, 0);
@@ -1299,7 +1412,7 @@ void wShowInfoPanel(WScreen * scr)
 	wWindowMap(wwin);
 
 	panel->wwin = wwin;
-	thePanel = panel;
+	infoPanel = panel;
 }
 
 /*
@@ -1310,11 +1423,9 @@ void wShowInfoPanel(WScreen * scr)
 
 typedef struct {
 	WScreen *scr;
-
 	WWindow *wwin;
-
 	WMWindow *win;
-
+	WMFrame *frame;
 	WMLabel *licenseL;
 } LegalPanel;
 
@@ -1322,22 +1433,26 @@ static LegalPanel *legalPanel = NULL;
 
 static void destroyLegalPanel(WCoreWindow * foo, void *data, XEvent * event)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) foo;
+	(void) data;
+	(void) event;
+
 	WMUnmapWidget(legalPanel->win);
-
 	WMDestroyWidget(legalPanel->win);
-
 	wUnmanageWindow(legalPanel->wwin, False, False);
-
 	wfree(legalPanel);
-
 	legalPanel = NULL;
 }
 
-void wShowLegalPanel(WScreen * scr)
+void wShowLegalPanel(WScreen *scr)
 {
 	LegalPanel *panel;
 	Window parent;
 	WWindow *wwin;
+	WMPoint center;
+	int wmScaleWidth, wmScaleHeight;
+	int pwidth, pheight;
 
 	if (legalPanel) {
 		if (legalPanel->scr == scr) {
@@ -1348,44 +1463,45 @@ void wShowLegalPanel(WScreen * scr)
 	}
 
 	panel = wmalloc(sizeof(LegalPanel));
-
 	panel->scr = scr;
-
 	panel->win = WMCreateWindow(scr->wmscreen, "legal");
-	WMResizeWidget(panel->win, 420, 250);
+	WMGetScaleBaseFromSystemFont(scr->wmscreen, &wmScaleWidth, &wmScaleHeight);
+	pwidth = WMScaleX(440);
+	pheight = WMScaleY(270);
+	WMResizeWidget(panel->win, pwidth, pheight);
 
-	panel->licenseL = WMCreateLabel(panel->win);
+	panel->frame = WMCreateFrame(panel->win);
+	WMResizeWidget(panel->frame, pwidth - (2 * WMScaleX(10)), pheight - (2 * WMScaleY(10)));
+	WMMoveWidget(panel->frame, WMScaleX(10), WMScaleY(10));
+	WMSetFrameTitle(panel->frame, NULL);
+
+	panel->licenseL = WMCreateLabel(panel->frame);
 	WMSetLabelWraps(panel->licenseL, True);
-	WMResizeWidget(panel->licenseL, 400, 230);
-	WMMoveWidget(panel->licenseL, 10, 10);
+	WMResizeWidget(panel->licenseL, pwidth - (4 * WMScaleX(10)), pheight - (4 * WMScaleY(10)));
+	WMMoveWidget(panel->licenseL, WMScaleX(8), WMScaleY(8));
 	WMSetLabelTextAlignment(panel->licenseL, WALeft);
 	WMSetLabelText(panel->licenseL,
-		       _("    Window Maker is free software; you can redistribute it and/or\n"
-			 "modify it under the terms of the GNU General Public License as\n"
-			 "published by the Free Software Foundation; either version 2 of the\n"
+		       _("    Window Maker is free software; you can redistribute it and/or "
+			 "modify it under the terms of the GNU General Public License as "
+			 "published by the Free Software Foundation; either version 2 of the "
 			 "License, or (at your option) any later version.\n\n"
-			 "    Window Maker is distributed in the hope that it will be useful,\n"
-			 "but WITHOUT ANY WARRANTY; without even the implied warranty\n"
-			 "of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"
+			 "    Window Maker is distributed in the hope that it will be useful, "
+			 "but WITHOUT ANY WARRANTY; without even the implied warranty "
+			 "of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. "
 			 "See the GNU General Public License for more details.\n\n"
-			 "    You should have received a copy of the GNU General Public\n"
-			 "License along with this program; if not, write to the Free Software\n"
-			 "Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA\n"
+			 "    You should have received a copy of the GNU General Public "
+			 "License along with this program; if not, write to the Free Software "
+			 "Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA"
 			 "02110-1301 USA."));
-	WMSetLabelRelief(panel->licenseL, WRGroove);
 
 	WMRealizeWidget(panel->win);
 	WMMapSubwidgets(panel->win);
+	WMMapSubwidgets(panel->frame);
 
-	parent = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, 420, 250, 0, 0, 0);
-
+	parent = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, pwidth, pheight, 0, 0, 0);
 	XReparentWindow(dpy, WMWidgetXID(panel->win), parent, 0, 0);
-
-	{
-		WMPoint center = getCenter(scr, 420, 250);
-
-		wwin = wManageInternalWindow(scr, parent, None, _("Legal"), center.x, center.y, 420, 250);
-	}
+	center = getCenter(scr, pwidth, pheight);
+	wwin = wManageInternalWindow(scr, parent, None, _("Legal"), center.x, center.y, pwidth, pheight);
 
 	WSETUFLAG(wwin, no_closable, 0);
 	WSETUFLAG(wwin, no_close_button, 0);
@@ -1395,13 +1511,10 @@ void wShowLegalPanel(WScreen * scr)
 	wFrameWindowHideButton(wwin->frame, WFF_LANGUAGE_BUTTON);
 #endif
 	wwin->frame->on_click_right = destroyLegalPanel;
-
 	panel->wwin = wwin;
 
 	WMMapWidget(panel->win);
-
 	wWindowMap(wwin);
-
 	legalPanel = panel;
 }
 
@@ -1411,8 +1524,6 @@ void wShowLegalPanel(WScreen * scr)
  ***********************************************************************
  */
 
-extern WDDomain *WDWindowAttributes;
-
 typedef struct _CrashPanel {
 	WMWindow *win;		/* main window */
 
@@ -1449,6 +1560,9 @@ static void okButtonCallback(void *self, void *clientData)
 {
 	CrashPanel *panel = (CrashPanel *) clientData;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) self;
+
 	panel->done = True;
 }
 
@@ -1461,45 +1575,24 @@ static void setCrashAction(void *self, void *clientData)
 }
 
 /* Make this read the logo from a compiled in pixmap -Dan */
-static WMPixmap *getWindowMakerIconImage(WMScreen * scr)
+static WMPixmap *getWindowMakerIconImage(WMScreen *scr)
 {
-	WMPropList *dict, *key, *option, *value = NULL;
 	WMPixmap *pix = NULL;
-	char *path;
+	char *path = NULL;
 
-	if (!WDWindowAttributes || !WDWindowAttributes->dictionary)
-		return NULL;
+	/* Get the Logo icon, without the default icon */
+	path = get_icon_filename("Logo", "WMPanel", NULL, False);
 
-	WMPLSetCaseSensitive(True);
+	if (path) {
+		RColor gray;
 
-	key = WMCreatePLString("Logo.WMPanel");
-	option = WMCreatePLString("Icon");
+		gray.red = 0xae;
+		gray.green = 0xaa;
+		gray.blue = 0xae;
+		gray.alpha = 0;
 
-	dict = WMGetFromPLDictionary(WDWindowAttributes->dictionary, key);
-
-	if (dict) {
-		value = WMGetFromPLDictionary(dict, option);
-	}
-
-	WMReleasePropList(key);
-	WMReleasePropList(option);
-
-	WMPLSetCaseSensitive(False);
-
-	if (value && WMIsPLString(value)) {
-		path = FindImage(wPreferences.icon_path, WMGetFromPLString(value));
-
-		if (path) {
-			RColor gray;
-
-			gray.red = 0xae;
-			gray.green = 0xaa;
-			gray.blue = 0xae;
-			gray.alpha = 0;
-
-			pix = WMCreateBlendedPixmapFromFile(scr, path, &gray);
-			wfree(path);
-		}
+		pix = WMCreateBlendedPixmapFromFile(scr, path, &gray);
+		wfree(path);
 	}
 
 	return pix;
@@ -1518,8 +1611,6 @@ int wShowCrashingDialogPanel(int whatSig)
 	int action;
 	char buf[256];
 
-	panel = wmalloc(sizeof(CrashPanel));
-
 	screen_no = DefaultScreen(dpy);
 	scr_width = WidthOfScreen(ScreenOfDisplay(dpy, screen_no));
 	scr_height = HeightOfScreen(ScreenOfDisplay(dpy, screen_no));
@@ -1530,6 +1621,8 @@ int wShowCrashingDialogPanel(int whatSig)
 		return WMAbort;
 	}
 
+	panel = wmalloc(sizeof(CrashPanel));
+
 	panel->retKey = XKeysymToKeycode(dpy, XK_Return);
 
 	panel->win = WMCreateWindow(scr, "crashingDialog");
@@ -1569,9 +1662,11 @@ int wShowCrashingDialogPanel(int whatSig)
 	WMResizeWidget(panel->note2L, PWIDTH - 20, 100);
 	WMMoveWidget(panel->note2L, 10, 130);
 	WMSetLabelTextAlignment(panel->note2L, WALeft);
-	WMSetLabelText(panel->note2L,
-		       _(" This fatal error occured probably due to a bug."
-			 " Please fill the included BUGFORM and " "report it to bugs@windowmaker.info."));
+	snprintf(buf, sizeof(buf), /* Comment for the PO file: the %s is an email address */
+	         _(" This fatal error occurred probably due to a bug."
+	           " Please fill the included BUGFORM and report it to %s."),
+	         PACKAGE_BUGREPORT);
+	WMSetLabelText(panel->note2L, buf);
 	WMSetLabelWraps(panel->note2L, True);
 
 	panel->whatF = WMCreateFrame(panel->win);
