@@ -38,82 +38,86 @@
 #include "window.h"
 #include "icon.h"
 #include "actions.h"
-#include "funcs.h"
 #include "stacking.h"
 #include "application.h"
 #include "defaults.h"
 #include "appicon.h"
 #include "wmspec.h"
+#include "misc.h"
+#include "startup.h"
+#include "event.h"
+#include "winmenu.h"
 
 /**** Global varianebles ****/
-extern WPreferences wPreferences;
 
 #define MOD_MASK wPreferences.modifier_mask
+#define CACHE_ICON_PATH "/Library/WindowMaker/CachedPixmaps"
+#define ICON_BORDER 3
 
-extern Cursor wCursor[WCUR_LAST];
+static void miniwindowExpose(WObjDescriptor *desc, XEvent *event);
+static void miniwindowMouseDown(WObjDescriptor *desc, XEvent *event);
+static void miniwindowDblClick(WObjDescriptor *desc, XEvent *event);
 
-static void miniwindowExpose(WObjDescriptor * desc, XEvent * event);
-static void miniwindowMouseDown(WObjDescriptor * desc, XEvent * event);
-static void miniwindowDblClick(WObjDescriptor * desc, XEvent * event);
+static WIcon *icon_create_core(WScreen *scr, int coord_x, int coord_y);
 
-static WIcon *wIconCreateCore(WScreen *scr, int coord_x, int coord_y);
+static void set_dockapp_in_icon(WIcon *icon);
+static void get_rimage_icon_from_icon_win(WIcon *icon);
+static void get_rimage_icon_from_user_icon(WIcon *icon);
+static void get_rimage_icon_from_default_icon(WIcon *icon);
+static void get_rimage_icon_from_x11(WIcon *icon);
+
+static void icon_update_pixmap(WIcon *icon, RImage *image);
+static void unset_icon_image(WIcon *icon);
 
-void get_pixmap_icon_from_icon_win(WIcon *icon);
-int get_pixmap_icon_from_wm_hints(WScreen *scr, WWindow *wwin, WIcon *icon);
-void get_pixmap_icon_from_user_icon(WScreen *scr, WIcon * icon);
 /****** Notification Observers ******/
 
-static void appearanceObserver(void *self, WMNotification * notif)
+static void appearanceObserver(void *self, WMNotification *notif)
 {
 	WIcon *icon = (WIcon *) self;
 	uintptr_t flags = (uintptr_t)WMGetNotificationClientData(notif);
 
-	if (flags & WTextureSettings) {
-		icon->force_paint = 1;
+	if ((flags & WTextureSettings) || (flags & WFontSettings)) {
+		/* If the rimage exists, update the icon, else create it */
+		if (icon->file_image)
+			update_icon_pixmap(icon);
+		else
+			wIconPaint(icon);
 	}
-	if (flags & WFontSettings) {
-		icon->force_paint = 1;
-	}
-	/*
-	   if (flags & WColorSettings) {
-	   }
-	 */
-
-	wIconPaint(icon);
 
 	/* so that the appicon expose handlers will paint the appicon specific
 	 * stuff */
 	XClearArea(dpy, icon->core->window, 0, 0, icon->core->width, icon->core->height, True);
 }
 
-static void tileObserver(void *self, WMNotification * notif)
+static void tileObserver(void *self, WMNotification *notif)
 {
 	WIcon *icon = (WIcon *) self;
 
-	icon->force_paint = 1;
-	wIconPaint(icon);
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) notif;
+
+	update_icon_pixmap(icon);
 
 	XClearArea(dpy, icon->core->window, 0, 0, 1, 1, True);
 }
 
 /************************************/
 
-INLINE static void getSize(Drawable d, unsigned int *w, unsigned int *h, unsigned int *dep)
+static int getSize(Drawable d, unsigned int *w, unsigned int *h, unsigned int *dep)
 {
 	Window rjunk;
 	int xjunk, yjunk;
 	unsigned int bjunk;
 
-	XGetGeometry(dpy, d, &rjunk, &xjunk, &yjunk, w, h, &bjunk, dep);
+	return XGetGeometry(dpy, d, &rjunk, &xjunk, &yjunk, w, h, &bjunk, dep);
 }
 
-WIcon *wIconCreate(WWindow * wwin)
+WIcon *icon_create_for_wwindow(WWindow *wwin)
 {
 	WScreen *scr = wwin->screen_ptr;
 	WIcon *icon;
-	char *file;
 
-	icon = wIconCreateCore(scr, wwin->icon_x, wwin->icon_y);
+	icon = icon_create_core(scr, wwin->icon_x, wwin->icon_y);
 
 	icon->owner = wwin;
 	if (wwin->wm_hints && (wwin->wm_hints->flags & IconWindowHint)) {
@@ -132,48 +136,29 @@ WIcon *wIconCreate(WWindow * wwin)
 #else
 	icon->show_title = 1;
 #endif
-	icon->file_image = wDefaultGetImage(scr, wwin->wm_instance, wwin->wm_class, wPreferences.icon_size);
-
-	file = wDefaultGetIconFile(scr, wwin->wm_instance, wwin->wm_class, False);
-	if (file)
-		icon->file = wstrdup(file);
-
-	icon->icon_name = wNETWMGetIconName(wwin->client_win);
-	if (icon->icon_name)
-		wwin->flags.net_has_icon_title = 1;
-	else
-		wGetIconName(dpy, wwin->client_win, &icon->icon_name);
 
+	wIconChangeTitle(icon, wwin);
 	icon->tile_type = TILE_NORMAL;
 
+	set_icon_image_from_database(icon, wwin->wm_instance, wwin->wm_class, NULL);
+	/* Update the icon, because icon could be NULL */
 	wIconUpdate(icon);
 
-	XFlush(dpy);
-
 	WMAddNotificationObserver(appearanceObserver, icon, WNIconAppearanceSettingsChanged, icon);
 	WMAddNotificationObserver(tileObserver, icon, WNIconTileSettingsChanged, icon);
+
 	return icon;
 }
 
-WIcon *wIconCreateWithIconFile(WScreen * scr, char *iconfile, int tile)
+WIcon *icon_create_for_dock(WScreen *scr, const char *command, const char *wm_instance, const char *wm_class, int tile)
 {
 	WIcon *icon;
 
-	icon = wIconCreateCore(scr, 0, 0);
-
-	if (iconfile) {
-		icon->file_image = RLoadImage(scr->rcontext, iconfile, 0);
-		if (!icon->file_image) {
-			wwarning(_("error loading image file \"%s\": %s"), iconfile, RMessageForError(RErrorCode));
-		}
-
-		icon->file_image = wIconValidateIconSize(scr, icon->file_image, wPreferences.icon_size);
-
-		icon->file = wstrdup(iconfile);
-	}
-
+	icon = icon_create_core(scr, 0, 0);
 	icon->tile_type = tile;
 
+	set_icon_image_from_database(icon, wm_instance, wm_class, command);
+	/* Update the icon, because icon could be NULL */
 	wIconUpdate(icon);
 
 	WMAddNotificationObserver(appearanceObserver, icon, WNIconAppearanceSettingsChanged, icon);
@@ -182,11 +167,9 @@ WIcon *wIconCreateWithIconFile(WScreen * scr, char *iconfile, int tile)
 	return icon;
 }
 
-static WIcon *wIconCreateCore(WScreen *scr, int coord_x, int coord_y)
+static WIcon *icon_create_core(WScreen *scr, int coord_x, int coord_y)
 {
 	WIcon *icon;
-	unsigned long vmask = 0;
-	XSetWindowAttributes attribs;
 
 	icon = wmalloc(sizeof(WIcon));
 	icon->core = wCoreCreateTopLevel(scr,
@@ -194,18 +177,8 @@ static WIcon *wIconCreateCore(WScreen *scr, int coord_x, int coord_y)
 					 coord_y,
 					 wPreferences.icon_size,
 					 wPreferences.icon_size,
-					 0);
-
-	if (wPreferences.use_saveunders) {
-		vmask = CWSaveUnder;
-		attribs.save_under = True;
-	}
-
-	/* a white border for selecting it */
-	vmask |= CWBorderPixel;
-	attribs.border_pixel = scr->white_pixel;
-
-	XChangeWindowAttributes(dpy, icon->core->window, vmask, &attribs);
+					 0, scr->w_depth, scr->w_visual, scr->w_colormap,
+					 scr->white_pixel);
 
 	/* will be overriden if this is a application icon */
 	icon->core->descriptor.handle_mousedown = miniwindowMouseDown;
@@ -219,10 +192,14 @@ static WIcon *wIconCreateCore(WScreen *scr, int coord_x, int coord_y)
 	icon->core->stacking->window_level = NORMAL_ICON_LEVEL;
 	icon->core->stacking->child_of = NULL;
 
+	/* Icon image */
+	icon->file = NULL;
+	icon->file_image = NULL;
+
 	return icon;
 }
 
-void wIconDestroy(WIcon * icon)
+void wIconDestroy(WIcon *icon)
 {
 	WCoreWindow *core = icon->core;
 	WScreen *scr = core->screen_ptr;
@@ -248,17 +225,16 @@ void wIconDestroy(WIcon * icon)
 	if (icon->pixmap)
 		XFreePixmap(dpy, icon->pixmap);
 
-	if (icon->file)
-		wfree(icon->file);
+	if (icon->mini_preview)
+		XFreePixmap(dpy, icon->mini_preview);
 
-	if (icon->file_image != NULL)
-		RReleaseImage(icon->file_image);
+	unset_icon_image(icon);
 
 	wCoreDestroy(icon->core);
 	wfree(icon);
 }
 
-static void drawIconTitle(WScreen * scr, Pixmap pixmap, int height)
+static void drawIconTitleBackground(WScreen *scr, Pixmap pixmap, int height)
 {
 	XFillRectangle(dpy, pixmap, scr->icon_title_texture->normal_gc, 0, 0, wPreferences.icon_size, height + 1);
 	XDrawLine(dpy, pixmap, scr->icon_title_texture->light_gc, 0, 0, wPreferences.icon_size, 0);
@@ -267,39 +243,53 @@ static void drawIconTitle(WScreen * scr, Pixmap pixmap, int height)
 		  wPreferences.icon_size - 1, 0, wPreferences.icon_size - 1, height + 1);
 }
 
-static Pixmap makeIcon(WScreen *scr, RImage *icon, int titled, int shadowed, int tileType, int highlighted)
+static void icon_update_pixmap(WIcon *icon, RImage *image)
 {
 	RImage *tile;
 	Pixmap pixmap;
 	int x, y, sx, sy;
 	unsigned w, h;
 	int theight = 0;
+	WScreen *scr = icon->core->screen_ptr;
 
-	if (tileType == TILE_NORMAL) {
+	switch (icon->tile_type) {
+	case TILE_NORMAL:
 		tile = RCloneImage(scr->icon_tile);
-	} else {
-		assert(scr->clip_tile);
+		break;
+	case TILE_CLIP:
 		tile = RCloneImage(scr->clip_tile);
+		break;
+	case TILE_DRAWER:
+		tile = RCloneImage(scr->drawer_tile);
+		break;
+	default:
+		/*
+		 * The icon has always rigth value, this case is
+		 * only to avoid a compiler warning with "tile"
+		 * "may be used uninitialized"
+		 */
+		wwarning("Unknown tile type: %d.\n", icon->tile_type);
+		tile = RCloneImage(scr->icon_tile);
 	}
 
-	if (icon) {
-		w = (icon->width > wPreferences.icon_size)
-		    ? wPreferences.icon_size : icon->width;
+	if (image) {
+		w = (image->width > wPreferences.icon_size)
+		    ? wPreferences.icon_size : image->width;
 		x = (wPreferences.icon_size - w) / 2;
-		sx = (icon->width - w) / 2;
+		sx = (image->width - w) / 2;
 
-		if (titled)
+		if (icon->show_title)
 			theight = WMFontHeight(scr->icon_title_font);
 
-		h = (icon->height + theight > wPreferences.icon_size
-		     ? wPreferences.icon_size - theight : icon->height);
+		h = (image->height + theight > wPreferences.icon_size
+		     ? wPreferences.icon_size - theight : image->height);
 		y = theight + (wPreferences.icon_size - theight - h) / 2;
-		sy = (icon->height - h) / 2;
+		sy = (image->height - h) / 2;
 
-		RCombineArea(tile, icon, sx, sy, w, h, x, y);
+		RCombineArea(tile, image, sx, sy, w, h, x, y);
 	}
 
-	if (shadowed) {
+	if (icon->shadowed) {
 		RColor color;
 
 		color.red = scr->icon_back_texture->light.red >> 8;
@@ -309,7 +299,7 @@ static Pixmap makeIcon(WScreen *scr, RImage *icon, int titled, int shadowed, int
 		RClearImage(tile, &color);
 	}
 
-	if (highlighted) {
+	if (icon->highlighted) {
 		RColor color;
 
 		color.red = color.green = color.blue = 0;
@@ -322,182 +312,239 @@ static Pixmap makeIcon(WScreen *scr, RImage *icon, int titled, int shadowed, int
 
 	RReleaseImage(tile);
 
-	if (titled)
-		drawIconTitle(scr, pixmap, theight);
+	/* Draw the icon's title background (without text) */
+	if (icon->show_title)
+		drawIconTitleBackground(scr, pixmap, theight);
 
-	return pixmap;
+	icon->pixmap = pixmap;
 }
 
-void wIconChangeTitle(WIcon * icon, char *new_title)
+void wIconChangeTitle(WIcon *icon, WWindow *wwin)
 {
-	int changed;
-
-	changed = (new_title == NULL && icon->icon_name != NULL)
-	    || (new_title != NULL && icon->icon_name == NULL);
+	if (!icon || !wwin)
+		return;
 
+	/* Remove the previous icon title */
 	if (icon->icon_name != NULL)
 		XFree(icon->icon_name);
 
-	icon->icon_name = new_title;
-
-	if (changed)
-		icon->force_paint = 1;
-	wIconPaint(icon);
+	/* Set the new one, using two methods to identify
+	the icon name or switch back to window name */
+	icon->icon_name = wNETWMGetIconName(wwin->client_win);
+	if (!icon->icon_name)
+		if (!wGetIconName(dpy, wwin->client_win, &icon->icon_name))
+			icon->icon_name = wNETWMGetWindowName(wwin->client_win);
 }
 
-RImage *wIconValidateIconSize(WScreen * scr, RImage * icon, int max_size)
+RImage *wIconValidateIconSize(RImage *icon, int max_size)
 {
-	RImage *tmp;
-	int w, h;
+	RImage *nimage;
 
 	if (!icon)
 		return NULL;
-#ifndef DONT_SCALE_ICONS
-	if (max_size != 64) {
-		w = max_size * icon->width / 64;
-		h = max_size * icon->height / 64;
 
-		tmp = RScaleImage(icon, w, h);
+	/* We should hold "ICON_BORDER" (~2) pixels to include the icon border */
+	if (((max_size + ICON_BORDER) < icon->width) ||
+	    ((max_size + ICON_BORDER) < icon->height)) {
+		if (icon->width > icon->height)
+			nimage = RScaleImage(icon, max_size - ICON_BORDER,
+					     (icon->height * (max_size - ICON_BORDER) / icon->width));
+		else
+			nimage = RScaleImage(icon, (icon->width * (max_size - ICON_BORDER) / icon->height),
+					     max_size - ICON_BORDER);
 		RReleaseImage(icon);
-		icon = tmp;
+		icon = nimage;
 	}
-#endif
 
 	return icon;
 }
 
-Bool wIconChangeImageFile(WIcon * icon, char *file)
+int wIconChangeImageFile(WIcon *icon, const char *file)
 {
 	WScreen *scr = icon->core->screen_ptr;
-	RImage *image;
 	char *path;
-	int error = 0;
+	RImage *image = NULL;
 
-	if (icon->file_image)
-		RReleaseImage(icon->file_image);
-
-	if (!file) {
-		icon->file_image = NULL;
-		wIconUpdate(icon);
-		return True;
-	}
+	/* If no new image, don't do nothing */
+	if (!file)
+		return 1;
 
+	/* Find the new image */
 	path = FindImage(wPreferences.icon_path, file);
+	if (!path)
+		return 0;
 
-	if (path && (image = RLoadImage(scr->rcontext, path, 0))) {
-		icon->file_image = wIconValidateIconSize(icon->core->screen_ptr, image, wPreferences.icon_size);
-		wIconUpdate(icon);
-	} else {
-		error = 1;
+	image = get_rimage_from_file(scr, path, wPreferences.icon_size);
+	if (!image) {
+		wfree(path);
+		return 0;
 	}
 
-	if (path)
-		wfree(path);
+	/* Set the new image */
+	set_icon_image_from_image(icon, image);
+	icon->file = wstrdup(path);
+	update_icon_pixmap(icon);
 
-	return !error;
+	wfree(path);
+	return 1;
 }
 
-static char *getnameforicon(WWindow * wwin)
+static char *get_name_for_wwin(WWindow *wwin)
 {
-	char *prefix, *suffix;
-	char *path;
+	return get_name_for_instance_class(wwin->wm_instance, wwin->wm_class);
+}
+
+char *get_name_for_instance_class(const char *wm_instance, const char *wm_class)
+{
+	char *suffix;
 	int len;
 
-	if (wwin->wm_class && wwin->wm_instance) {
-		int len = strlen(wwin->wm_class) + strlen(wwin->wm_instance) + 2;
+	if (wm_class && wm_instance) {
+		len = strlen(wm_class) + strlen(wm_instance) + 2;
 		suffix = wmalloc(len);
-		snprintf(suffix, len, "%s.%s", wwin->wm_instance, wwin->wm_class);
-	} else if (wwin->wm_class) {
-		int len = strlen(wwin->wm_class) + 1;
+		snprintf(suffix, len, "%s.%s", wm_instance, wm_class);
+	} else if (wm_class) {
+		len = strlen(wm_class) + 1;
 		suffix = wmalloc(len);
-		snprintf(suffix, len, "%s", wwin->wm_class);
-	} else if (wwin->wm_instance) {
-		int len = strlen(wwin->wm_instance) + 1;
+		snprintf(suffix, len, "%s", wm_class);
+	} else if (wm_instance) {
+		len = strlen(wm_instance) + 1;
 		suffix = wmalloc(len);
-		snprintf(suffix, len, "%s", wwin->wm_instance);
+		snprintf(suffix, len, "%s", wm_instance);
 	} else {
 		return NULL;
 	}
 
+	return suffix;
+}
+
+static char *get_icon_cache_path(void)
+{
+	const char *prefix;
+	char *path;
+	int len, ret;
+
 	prefix = wusergnusteppath();
-	len = strlen(prefix) + 64 + strlen(suffix);
-	path = wmalloc(len + 1);
-	snprintf(path, len, "%s/Library/WindowMaker", prefix);
+	len = strlen(prefix) + strlen(CACHE_ICON_PATH) + 2;
+	path = wmalloc(len);
+	snprintf(path, len, "%s%s/", prefix, CACHE_ICON_PATH);
 
-	if (access(path, F_OK) != 0) {
-		if (mkdir(path, S_IRUSR | S_IWUSR | S_IXUSR)) {
-			werror(_("could not create directory %s"), path);
-			wfree(path);
-			wfree(suffix);
-			return NULL;
-		}
-	}
-	strcat(path, "/CachedPixmaps");
-	if (access(path, F_OK) != 0) {
-		if (mkdir(path, S_IRUSR | S_IWUSR | S_IXUSR) != 0) {
-			werror(_("could not create directory %s"), path);
-			wfree(path);
-			wfree(suffix);
-			return NULL;
-		}
-	}
+	/* If the folder exists, exit */
+	if (access(path, F_OK) == 0)
+		return path;
 
-	strcat(path, "/");
-	strcat(path, suffix);
-	strcat(path, ".xpm");
-	wfree(suffix);
+	/* Create the folder */
+	ret = wmkdirhier((const char *) path);
 
-	return path;
+	/* Exit 1 on success, 0 on failure */
+	if (ret == 1)
+		return path;
+
+	/* Fail */
+	wfree(path);
+	return NULL;
+}
+
+static RImage *get_wwindow_image_from_wmhints(WWindow *wwin, WIcon *icon)
+{
+	RImage *image = NULL;
+	XWMHints *hints = wwin->wm_hints;
+
+	if (hints && (hints->flags & IconPixmapHint) && hints->icon_pixmap != None)
+		image = RCreateImageFromDrawable(icon->core->screen_ptr->rcontext,
+						 hints->icon_pixmap,
+						 (hints->flags & IconMaskHint)
+						 ? hints->icon_mask : None);
+
+	return image;
 }
 
 /*
  * wIconStore--
- * 	Stores the client supplied icon at ~/GNUstep/Library/WindowMaker/CachedPixmaps
+ * 	Stores the client supplied icon at CACHE_ICON_PATH
  * and returns the path for that icon. Returns NULL if there is no
  * client supplied icon or on failure.
  *
  * Side effects:
  * 	New directories might be created.
  */
-char *wIconStore(WIcon * icon)
+char *wIconStore(WIcon *icon)
 {
-	char *path;
+	char *path, *dir_path, *file, *filename;
+	int len = 0;
 	RImage *image = NULL;
 	WWindow *wwin = icon->owner;
 
 	if (!wwin)
 		return NULL;
 
-	path = getnameforicon(wwin);
-	if (!path)
+	dir_path = get_icon_cache_path();
+	if (!dir_path)
 		return NULL;
 
-	/* If icon exists, exit */
-	if (access(path, F_OK) == 0)
-                return path;
-
-	if (wwin->net_icon_image) {
-		image = RRetainImage(wwin->net_icon_image);
-	} else if (wwin->wm_hints && (wwin->wm_hints->flags & IconPixmapHint)
-	    && wwin->wm_hints->icon_pixmap != None) {
-		image = RCreateImageFromDrawable(icon->core->screen_ptr->rcontext,
-						 wwin->wm_hints->icon_pixmap, (wwin->wm_hints->flags & IconMaskHint)
-						 ? wwin->wm_hints->icon_mask : None);
+	file = get_name_for_wwin(wwin);
+	if (!file) {
+		wfree(dir_path);
+		return NULL;
 	}
 
+	/* Create the file name */
+	len = strlen(file) + 5;
+	filename = wmalloc(len);
+	snprintf(filename, len, "%s.xpm", file);
+	wfree(file);
+
+	/* Create the full path, including the filename */
+	len = strlen(dir_path) + strlen(filename) + 1;
+	path = wmalloc(len);
+	snprintf(path, len, "%s%s", dir_path, filename);
+	wfree(dir_path);
+
+	/* If icon exists, exit */
+	if (access(path, F_OK) == 0) {
+		wfree(path);
+		return filename;
+	}
+
+	if (wwin->net_icon_image)
+		image = RRetainImage(wwin->net_icon_image);
+	else
+		image = get_wwindow_image_from_wmhints(wwin, icon);
+
 	if (!image) {
 		wfree(path);
+		wfree(filename);
 		return NULL;
 	}
 
 	if (!RSaveImage(image, path, "XPM")) {
 		wfree(path);
+		wfree(filename);
 		path = NULL;
 	}
 
+	wfree(path);
 	RReleaseImage(image);
 
-	return path;
+	return filename;
+}
+
+void remove_cache_icon(char *filename)
+{
+	char *cachepath;
+
+	if (!filename)
+		return;
+
+	cachepath = get_icon_cache_path();
+	if (!cachepath)
+		return;
+
+	/* Filename check/parse could be here */
+	if (!strncmp(filename, cachepath, strlen(cachepath)))
+		unlink(filename);
+
+	wfree(cachepath);
 }
 
 static void cycleColor(void *data)
@@ -515,19 +562,16 @@ static void cycleColor(void *data)
 	icon->handlerID = WMAddTimerHandler(COLOR_CYCLE_DELAY, cycleColor, icon);
 }
 
-#ifdef NEWAPPICON
 void wIconSetHighlited(WIcon *icon, Bool flag)
 {
 	if (icon->highlighted == flag)
 		return;
 
 	icon->highlighted = flag;
-	icon->force_paint = True;
-	wIconPaint(icon);
+	update_icon_pixmap(icon);
 }
-#endif
 
-void wIconSelect(WIcon * icon)
+void wIconSelect(WIcon *icon)
 {
 	WScreen *scr = icon->core->screen_ptr;
 	icon->selected = !icon->selected;
@@ -548,35 +592,93 @@ void wIconSelect(WIcon * icon)
 	}
 }
 
-void wIconUpdate(WIcon * icon)
+static void unset_icon_image(WIcon *icon)
 {
+	if (icon->file) {
+		wfree(icon->file);
+		icon->file = NULL;
+	}
+
+	if (icon->file_image) {
+		RReleaseImage(icon->file_image);
+		icon->file_image = NULL;
+	}
+}
+
+void set_icon_image_from_image(WIcon *icon, RImage *image)
+{
+	if (!icon)
+		return;
+
+	unset_icon_image(icon);
+
+	icon->file_image = NULL;
+	icon->file_image = image;
+}
+
+void set_icon_minipreview(WIcon *icon, RImage *image)
+{
+	Pixmap tmp;
+	RImage *scaled_mini_preview;
 	WScreen *scr = icon->core->screen_ptr;
-	WWindow *wwin = icon->owner;
 
-	assert(scr->icon_tile != NULL);
+	scaled_mini_preview = RSmoothScaleImage(image, wPreferences.minipreview_size - 2 * MINIPREVIEW_BORDER,
+	                                  wPreferences.minipreview_size - 2 * MINIPREVIEW_BORDER);
 
+	if (RConvertImage(scr->rcontext, scaled_mini_preview, &tmp)) {
+		if (icon->mini_preview != None)
+			XFreePixmap(dpy, icon->mini_preview);
+		icon->mini_preview = tmp;
+	}
+	RReleaseImage(scaled_mini_preview);
+}
+
+void wIconUpdate(WIcon *icon)
+{
+	WWindow *wwin = NULL;
+
+	if (icon && icon->owner)
+		wwin = icon->owner;
+
+	if (wwin && WFLAGP(wwin, always_user_icon)) {
+		/* Forced use user_icon */
+		get_rimage_icon_from_user_icon(icon);
+	} else if (icon->icon_win != None) {
+		/* Get the Pixmap from the WIcon */
+		get_rimage_icon_from_icon_win(icon);
+	} else if (wwin && wwin->net_icon_image) {
+		/* Use _NET_WM_ICON icon */
+		get_rimage_icon_from_x11(icon);
+	} else if (wwin && wwin->wm_hints && (wwin->wm_hints->flags & IconPixmapHint)) {
+		/* Get the Pixmap from the wm_hints, else, from the user */
+		unset_icon_image(icon);
+		icon->file_image = get_rimage_icon_from_wm_hints(icon);
+		if (!icon->file_image)
+			get_rimage_icon_from_user_icon(icon);
+	} else {
+		/* Get the Pixmap from the user */
+		get_rimage_icon_from_user_icon(icon);
+	}
+
+	update_icon_pixmap(icon);
+}
+
+void update_icon_pixmap(WIcon *icon)
+{
 	if (icon->pixmap != None)
 		XFreePixmap(dpy, icon->pixmap);
 
 	icon->pixmap = None;
+ 
+	/* Create the pixmap */
+	if (icon->file_image)
+		icon_update_pixmap(icon, icon->file_image);
 
-	if (wwin && WFLAGP(wwin, always_user_icon)) {
-		/* Forced use user_icon */
-		get_pixmap_icon_from_user_icon(scr, icon);
-	} else if (icon->icon_win != None) {
-		/* Get the Pixmap from the WIcon */
-		get_pixmap_icon_from_icon_win(icon);
-	} else if (wwin && wwin->net_icon_image) {
-		/* Use _NET_WM_ICON icon */
-		icon->pixmap = makeIcon(scr, wwin->net_icon_image, icon->show_title,
-						icon->shadowed, icon->tile_type, icon->highlighted);
-	} else if (wwin && wwin->wm_hints && (wwin->wm_hints->flags & IconPixmapHint)) {
-		/* Get the Pixmap from the wm_hints, else, from the user */
-		if (get_pixmap_icon_from_wm_hints(scr, wwin, icon))
-			get_pixmap_icon_from_user_icon(scr, icon);
-	} else {
-		/* Get the Pixmap from the user */
-		get_pixmap_icon_from_user_icon(scr, icon);
+	/* If dockapp, put inside the icon */
+	if (icon->icon_win != None) {
+		/* file_image is NULL, because is docked app */
+		icon_update_pixmap(icon, NULL);
+		set_dockapp_in_icon(icon);
 	}
 
 	/* No pixmap, set default background */
@@ -584,187 +686,128 @@ void wIconUpdate(WIcon * icon)
 		XSetWindowBackgroundPixmap(dpy, icon->core->window, icon->pixmap);
 
 	/* Paint it */
-	XClearWindow(dpy, icon->core->window);
 	wIconPaint(icon);
 }
 
-void get_pixmap_icon_from_user_icon(WScreen *scr, WIcon * icon)
+static void get_rimage_icon_from_x11(WIcon *icon)
 {
-	RImage *image = NULL;
-	char *path;
-	char *file;
+	/* Remove the icon image */
+	unset_icon_image(icon);
 
-	if (icon->file_image) {
-		icon->pixmap = makeIcon(scr, icon->file_image, icon->show_title,
-					icon->shadowed, icon->tile_type, icon->highlighted);
-	} else {
-		/* make default icons */
-		if (!scr->def_icon_pixmap) {
-			file = wDefaultGetIconFile(scr, NULL, NULL, False);
-			if (file) {
-				path = FindImage(wPreferences.icon_path, file);
-				if (path) {
-					image = RLoadImage(scr->rcontext, path, 0);
-					if (!image)
-						wwarning(_("could not load default icon \"%s\":%s"),
-							 file, RMessageForError(RErrorCode));
-					wfree(path);
-				} else {
-					wwarning(_("could not find default icon \"%s\""), file);
-				}
-				/* FIXME: Probably wfree(file) here! */
-			}
-
-			image = wIconValidateIconSize(scr, image, wPreferences.icon_size);
-			scr->def_icon_pixmap = makeIcon(scr, image, False, False, icon->tile_type, icon->highlighted);
-			scr->def_ticon_pixmap = makeIcon(scr, image, True, False, icon->tile_type, icon->highlighted);
-			if (image)
-				RReleaseImage(image);
-		}
-
-		if (icon->show_title)
-			XSetWindowBackgroundPixmap(dpy, icon->core->window, scr->def_ticon_pixmap);
-		else
-			XSetWindowBackgroundPixmap(dpy, icon->core->window, scr->def_icon_pixmap);
-
-		icon->pixmap = None;
-	}
+	/* Set the new icon image */
+	icon->file_image = RRetainImage(icon->owner->net_icon_image);
 }
 
-/* Get the Pixmap from the WIcon of the WWindow */
-void get_pixmap_icon_from_icon_win(WIcon * icon)
+static void get_rimage_icon_from_user_icon(WIcon *icon)
+{
+	if (icon->file_image)
+		return;
+
+	get_rimage_icon_from_default_icon(icon);
+}
+
+static void get_rimage_icon_from_default_icon(WIcon *icon)
+{
+	WScreen *scr = icon->core->screen_ptr;
+
+	/* If the icon don't have image, we should use the default image. */
+	if (!scr->def_icon_rimage)
+		scr->def_icon_rimage = get_default_image(scr);
+
+	/* Remove the icon image */
+	unset_icon_image(icon);
+
+	/* Set the new icon image */
+	icon->file_image = RRetainImage(scr->def_icon_rimage);
+}
+
+/* Get the RImage from the WIcon of the WWindow */
+static void get_rimage_icon_from_icon_win(WIcon *icon)
+{
+	RImage *image;
+
+	/* Create the new RImage */
+	image = get_window_image_from_x11(icon->icon_win);
+
+	/* Free the icon info */
+	unset_icon_image(icon);
+
+	/* Set the new info */
+	icon->file_image = image;
+}
+
+/* Set the dockapp in the WIcon */
+static void set_dockapp_in_icon(WIcon *icon)
 {
 	XWindowAttributes attr;
 	WScreen *scr = icon->core->screen_ptr;
-	int title_height = WMFontHeight(scr->icon_title_font);
-	unsigned int width, height, depth;
-	int theight;
-	int resize = 0;
-	Pixmap pixmap;
+	unsigned int w, h, d;
 
-	getSize(icon->icon_win, &width, &height, &depth);
+	/* Reparent the dock application to the icon */
 
-	if (width > wPreferences.icon_size) {
-		resize = 1;
-		width = wPreferences.icon_size;
-	}
+	/* We need the application size to center it
+	 * and show in the correct position */
+	getSize(icon->icon_win, &w, &h, &d);
 
-	if (height > wPreferences.icon_size) {
-		resize = 1;
-		height = wPreferences.icon_size;
-	}
-
-	if (icon->show_title && (height + title_height < wPreferences.icon_size)) {
-		pixmap = XCreatePixmap(dpy, scr->w_win, wPreferences.icon_size,
-				       wPreferences.icon_size, scr->w_depth);
-		XSetClipMask(dpy, scr->copy_gc, None);
-		XCopyArea(dpy, scr->icon_tile_pixmap, pixmap, scr->copy_gc, 0, 0,
-			  wPreferences.icon_size, wPreferences.icon_size, 0, 0);
-		drawIconTitle(scr, pixmap, title_height);
-		theight = title_height;
-	} else {
-		pixmap = None;
-		theight = 0;
-		XSetWindowBackgroundPixmap(dpy, icon->core->window, scr->icon_tile_pixmap);
-	}
+	/* Set the background pixmap */
+	XSetWindowBackgroundPixmap(dpy, icon->core->window, scr->icon_tile_pixmap);
 
+	/* Set the icon border */
 	XSetWindowBorderWidth(dpy, icon->icon_win, 0);
-	XReparentWindow(dpy, icon->icon_win, icon->core->window,
-			(wPreferences.icon_size - width) / 2,
-			theight + (wPreferences.icon_size - height - theight) / 2);
-	if (resize)
-		XResizeWindow(dpy, icon->icon_win, width, height);
 
+	/* Put the dock application in the icon */
+	XReparentWindow(dpy, icon->icon_win, icon->core->window,
+			(wPreferences.icon_size - w) / 2,
+			(wPreferences.icon_size - h) / 2);
+
+	/* Show it and save */
 	XMapWindow(dpy, icon->icon_win);
 	XAddToSaveSet(dpy, icon->icon_win);
 
-	/* Save it */
-	icon->pixmap = pixmap;
-
+	/* Needed to move the icon clicking on the application part */
 	if ((XGetWindowAttributes(dpy, icon->icon_win, &attr)) &&
 	    (attr.all_event_masks & ButtonPressMask))
-			wHackedGrabButton(Button1, MOD_MASK, icon->core->window, True,
-					  ButtonPressMask, GrabModeSync, GrabModeAsync,
-					  None, wCursor[WCUR_ARROW]);
+		wHackedGrabButton(Button1, MOD_MASK, icon->core->window, True,
+				  ButtonPressMask, GrabModeSync, GrabModeAsync,
+				  None, wPreferences.cursor[WCUR_ARROW]);
 }
 
-/* Get the Pixmap from the XWindow wm_hints */
-int get_pixmap_icon_from_wm_hints(WScreen *scr, WWindow *wwin, WIcon *icon)
+/* Get the RImage from the XWindow wm_hints */
+RImage *get_rimage_icon_from_wm_hints(WIcon *icon)
 {
-	Window jw;
-	Pixmap pixmap;
-	unsigned int w, h, ju, d;
-	int ji, x, y;
-	int title_height = WMFontHeight(scr->icon_title_font);
+	RImage *image = NULL;
+	unsigned int w, h, d;
+	WWindow *wwin;
 
-	if (!XGetGeometry(dpy, wwin->wm_hints->icon_pixmap, &jw, &ji, &ji, &w, &h, &ju, &d)) {
+	if ((!icon) || (!icon->owner))
+		return NULL;
+
+	wwin = icon->owner;
+
+	if (!getSize(wwin->wm_hints->icon_pixmap, &w, &h, &d)) {
 		icon->owner->wm_hints->flags &= ~IconPixmapHint;
-		return(1);
+		return NULL;
 	}
 
-	pixmap = XCreatePixmap(dpy, icon->core->window, wPreferences.icon_size,
-			       wPreferences.icon_size, scr->w_depth);
-	XSetClipMask(dpy, scr->copy_gc, None);
-	XCopyArea(dpy, scr->icon_tile_pixmap, pixmap, scr->copy_gc, 0, 0,
-		  wPreferences.icon_size, wPreferences.icon_size, 0, 0);
+	image = get_wwindow_image_from_wmhints(wwin, icon);
+	if (!image)
+		return NULL;
 
-	if (w > wPreferences.icon_size)
-		w = wPreferences.icon_size;
-	x = (wPreferences.icon_size - w) / 2;
+	/* Resize the icon to the wPreferences.icon_size size */
+	image = wIconValidateIconSize(image, wPreferences.icon_size);
 
-	if (icon->show_title && (title_height < wPreferences.icon_size)) {
-		drawIconTitle(scr, pixmap, title_height);
-
-		if (h > wPreferences.icon_size - title_height - 2) {
-			h = wPreferences.icon_size - title_height - 2;
-			y = title_height + 1;
-		} else {
-			y = (wPreferences.icon_size - h - title_height) / 2 + title_height + 1;
-		}
-	} else {
-		if (w > wPreferences.icon_size)
-			w = wPreferences.icon_size;
-		y = (wPreferences.icon_size - h) / 2;
-	}
-
-	if (wwin->wm_hints->flags & IconMaskHint)
-		XSetClipMask(dpy, scr->copy_gc, wwin->wm_hints->icon_mask);
-
-	XSetClipOrigin(dpy, scr->copy_gc, x, y);
-
-	if (d != scr->w_depth) {
-		XSetForeground(dpy, scr->copy_gc, scr->black_pixel);
-		XSetBackground(dpy, scr->copy_gc, scr->white_pixel);
-		XCopyPlane(dpy, wwin->wm_hints->icon_pixmap, pixmap, scr->copy_gc, 0, 0, w, h, x, y, 1);
-	} else {
-		XCopyArea(dpy, wwin->wm_hints->icon_pixmap, pixmap, scr->copy_gc, 0, 0, w, h, x, y);
-	}
-
-	XSetClipOrigin(dpy, scr->copy_gc, 0, 0);
-
-	icon->pixmap = pixmap;
-	return (0);
+	return image;
 }
 
-void wIconPaint(WIcon * icon)
+/* This function updates in the screen the icon title */
+static void update_icon_title(WIcon *icon)
 {
 	WScreen *scr = icon->core->screen_ptr;
-	int x;
+	int x, l, w;
 	char *tmp;
 
-	if (icon->force_paint) {
-		icon->force_paint = 0;
-		wIconUpdate(icon);
-		return;
-	}
-
-	XClearWindow(dpy, icon->core->window);
-
 	/* draw the icon title */
 	if (icon->show_title && icon->icon_name != NULL) {
-		int l;
-		int w;
-
 		tmp = ShrinkString(scr->icon_title_font, icon->icon_name, wPreferences.icon_size - 4);
 		w = WMWidthOfString(scr->icon_title_font, tmp, l = strlen(tmp));
 
@@ -777,6 +820,19 @@ void wIconPaint(WIcon * icon)
 			     scr->icon_title_font, x, 1, tmp, l);
 		wfree(tmp);
 	}
+}
+
+
+void wIconPaint(WIcon *icon)
+{
+	if (!icon || !icon->core || !icon->core->screen_ptr)
+		return;
+
+	WScreen *scr = icon->core->screen_ptr;
+
+	XClearWindow(dpy, icon->core->window);
+
+	update_icon_title(icon);
 
 	if (icon->selected)
 		XDrawRectangle(dpy, icon->core->window, scr->icon_select_gc, 0, 0,
@@ -785,21 +841,27 @@ void wIconPaint(WIcon * icon)
 
 /******************************************************************/
 
-static void miniwindowExpose(WObjDescriptor * desc, XEvent * event)
+static void miniwindowExpose(WObjDescriptor *desc, XEvent *event)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) event;
+
 	wIconPaint(desc->parent);
 }
 
-static void miniwindowDblClick(WObjDescriptor * desc, XEvent * event)
+static void miniwindowDblClick(WObjDescriptor *desc, XEvent *event)
 {
 	WIcon *icon = desc->parent;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) event;
+
 	assert(icon->owner != NULL);
 
 	wDeiconifyWindow(icon->owner);
 }
 
-static void miniwindowMouseDown(WObjDescriptor * desc, XEvent * event)
+static void miniwindowMouseDown(WObjDescriptor *desc, XEvent *event)
 {
 	WIcon *icon = desc->parent;
 	WWindow *wwin = icon->owner;
@@ -859,7 +921,7 @@ static void miniwindowMouseDown(WObjDescriptor * desc, XEvent * event)
 				    || abs(dy - ev.xmotion.y) >= MOVE_THRESHOLD) {
 					XChangeActivePointerGrab(dpy, ButtonMotionMask
 								 | ButtonReleaseMask | ButtonPressMask,
-								 wCursor[WCUR_MOVE], CurrentTime);
+								 wPreferences.cursor[WCUR_MOVE], CurrentTime);
 					grabbed = 1;
 				} else {
 					break;
@@ -895,3 +957,15 @@ static void miniwindowMouseDown(WObjDescriptor * desc, XEvent * event)
 		}
 	}
 }
+
+void set_icon_image_from_database(WIcon *icon, const char *wm_instance, const char *wm_class, const char *command)
+{
+	char *file = NULL;
+
+	file = get_icon_filename(wm_instance, wm_class, command, False);
+	if (file) {
+		icon->file = wstrdup(file);
+		icon->file_image = get_rimage_from_file(icon->core->screen_ptr, icon->file, wPreferences.icon_size);
+		wfree(file);
+	}
+}
