@@ -37,34 +37,25 @@
 #include "framewin.h"
 #include "menu.h"
 #include "actions.h"
-#include "funcs.h"
+#include "winmenu.h"
 #include "stacking.h"
 #include "xinerama.h"
 #include "workspace.h"
 #include "dialog.h"
+#include "rootmenu.h"
+#include "switchmenu.h"
 
-/****** Global Variables ******/
-
-extern Cursor wCursor[WCUR_LAST];
-
-extern XContext wWinContext;
-
-extern WPreferences wPreferences;
 
 #define MOD_MASK wPreferences.modifier_mask
 
 #define MENU_SCROLL_STEP  menuScrollParameters[(int)wPreferences.menu_scroll_speed].steps
 #define MENU_SCROLL_DELAY menuScrollParameters[(int)wPreferences.menu_scroll_speed].delay
 
-#define MENUW(m)	((m)->frame->core->width+2*FRAME_BORDER_WIDTH)
-#define MENUH(m)	((m)->frame->core->height+2*FRAME_BORDER_WIDTH)
+#define MENUW(m)	((m)->frame->core->width+2*(m)->frame->screen_ptr->frame_border_width)
+#define MENUH(m)	((m)->frame->core->height+2*(m)->frame->screen_ptr->frame_border_width)
 
 /***** Local Stuff ******/
 
-#define WSS_ROOTMENU	(1<<0)
-#define WSS_SWITCHMENU	(1<<1)
-#define WSS_WSMENU	(1<<2)
-
 static struct {
 	int steps;
 	int delay;
@@ -83,7 +74,7 @@ static void menuTitleMouseDown(WCoreWindow * sender, void *data, XEvent * event)
 static void menuCloseClick(WCoreWindow * sender, void *data, XEvent * event);
 static void updateTexture(WMenu * menu);
 static int saveMenuRecurs(WMPropList * menus, WScreen * scr, WMenu * menu);
-static int restoreMenuRecurs(WScreen * scr, WMPropList * menus, WMenu * menu, char *path);
+static int restoreMenuRecurs(WScreen *scr, WMPropList *menus, WMenu *menu, const char *path);
 static void selectEntry(WMenu * menu, int entry_no);
 static void closeCascade(WMenu * menu);
 
@@ -138,7 +129,7 @@ static void appearanceObserver(void *self, WMNotification * notif)
  * 	The created menu.
  *----------------------------------------------------------------------
  */
-WMenu *wMenuCreate(WScreen * screen, char *title, int main_menu)
+WMenu *wMenuCreate(WScreen *screen, const char *title, int main_menu)
 {
 	WMenu *menu;
 	static int brother = 0;
@@ -163,7 +154,8 @@ WMenu *wMenuCreate(WScreen * screen, char *title, int main_menu)
 			       &wPreferences.menu_title_max_height,
 			       flags,
 			       screen->menu_title_texture, NULL,
-			       screen->menu_title_color, &screen->menu_title_font);
+			       screen->menu_title_color, &screen->menu_title_font,
+			       screen->w_depth, screen->w_visual, screen->w_colormap);
 
 	menu->frame->core->descriptor.parent = menu;
 	menu->frame->core->descriptor.parent_type = WCLASS_MENU;
@@ -190,6 +182,7 @@ WMenu *wMenuCreate(WScreen * screen, char *title, int main_menu)
 	menu->frame->child = menu;
 
 	menu->flags.lowered = 0;
+	menu->flags.shaded = 0;
 
 	/* create borders */
 	if (title) {
@@ -227,7 +220,7 @@ WMenu *wMenuCreate(WScreen * screen, char *title, int main_menu)
 	return menu;
 }
 
-WMenu *wMenuCreateForApp(WScreen * screen, char *title, int main_menu)
+WMenu *wMenuCreateForApp(WScreen *screen, const char *title, int main_menu)
 {
 	WMenu *menu;
 
@@ -251,7 +244,7 @@ static void insertEntry(WMenu * menu, WMenuEntry * entry, int index)
 	menu->entries[index] = entry;
 }
 
-WMenuEntry *wMenuInsertCallback(WMenu * menu, int index, char *text,
+WMenuEntry *wMenuInsertCallback(WMenu *menu, int index, const char *text,
 				void (*callback) (WMenu * menu, WMenuEntry * entry), void *clientdata)
 {
 	WMenuEntry *entry;
@@ -320,10 +313,10 @@ void wMenuEntrySetCascade(WMenu * menu, WMenuEntry * entry, WMenu * cascade)
 	if (!done) {
 		entry->cascade = menu->cascade_no;
 
-		menu->cascades = wrealloc(menu->cascades, sizeof(WMenu) * (menu->cascade_no + 1));
+		menu->cascades = wrealloc(menu->cascades, sizeof(menu->cascades[0]) * (menu->cascade_no + 1));
 		menu->cascades[menu->cascade_no++] = cascade;
 
-		brother->cascades = wrealloc(brother->cascades, sizeof(WMenu) * (brother->cascade_no + 1));
+		brother->cascades = wrealloc(brother->cascades, sizeof(brother->cascades[0]) * (brother->cascade_no + 1));
 		brother->cascades[brother->cascade_no++] = cascade->brother;
 	}
 
@@ -1060,15 +1053,16 @@ static int keyboardMenu(WMenu * menu)
 
 void wMenuMapAt(WMenu * menu, int x, int y, int keyboard)
 {
-	WMRect rect = wGetRectForHead(menu->frame->screen_ptr,
-				      wGetHeadForPointerLocation(menu->frame->screen_ptr));
-
 	if (!menu->flags.realized) {
 		menu->flags.realized = 1;
 		wMenuRealize(menu);
 	}
+
 	if (!menu->flags.mapped) {
 		if (wPreferences.wrap_menus) {
+			WScreen *scr = menu->frame->screen_ptr;
+			WMRect rect = wGetRectForHead(scr, wGetHeadForPointerLocation(scr));
+
 			if (x < rect.pos.x)
 				x = rect.pos.x;
 			if (y < rect.pos.y)
@@ -1121,6 +1115,12 @@ void wMenuUnmap(WMenu * menu)
 	menu->flags.mapped = 0;
 	menu->flags.open_to_left = 0;
 
+	if (menu->flags.shaded) {
+		wFrameWindowResize(menu->frame, menu->frame->core->width, menu->frame->top_width +
+				   menu->entry_height*menu->entry_no + menu->frame->bottom_width - 1);
+		menu->flags.shaded = 0;
+	}
+
 	for (i = 0; i < menu->cascade_no; i++) {
 		if (menu->cascades[i] != NULL
 		    && menu->cascades[i]->flags.mapped && !menu->cascades[i]->flags.buttoned) {
@@ -1276,7 +1276,7 @@ static WMenu *findMenu(WScreen * scr, int *x_ret, int *y_ret)
 	if (win == None)
 		return NULL;
 
-	if (XFindContext(dpy, win, wWinContext, (XPointer *) & desc) == XCNOENT)
+	if (XFindContext(dpy, win, w_global.context.client_win, (XPointer *) & desc) == XCNOENT)
 		return NULL;
 
 	if (desc->parent_type == WCLASS_MENU) {
@@ -1398,7 +1398,7 @@ WMenu *wMenuUnderPointer(WScreen * screen)
 	if (win == None)
 		return NULL;
 
-	if (XFindContext(dpy, win, wWinContext, (XPointer *) & desc) == XCNOENT)
+	if (XFindContext(dpy, win, w_global.context.client_win, (XPointer *) & desc) == XCNOENT)
 		return NULL;
 
 	if (desc->parent_type == WCLASS_MENU)
@@ -1519,7 +1519,7 @@ static void scrollMenuCallback(void *data)
 
 #define MENU_SCROLL_BORDER   5
 
-static int isPointNearBoder(WMenu * menu, int x, int y)
+static int isPointNearBorder(WMenu * menu, int x, int y)
 {
 	int menuX1 = menu->frame_x;
 	int menuY1 = menu->frame_y;
@@ -1546,15 +1546,17 @@ typedef struct _delay {
 	int ox, oy;
 } _delay;
 
-static void leaving(_delay * dl)
+static void callback_leaving(void *user_param)
 {
+	_delay *dl = (_delay *) user_param;
+
 	wMenuMove(dl->menu, dl->ox, dl->oy, True);
 	dl->menu->jump_back = NULL;
 	dl->menu->menu->screen_ptr->flags.jump_back_pending = 0;
 	wfree(dl);
 }
 
-void wMenuScroll(WMenu * menu, XEvent * event)
+void wMenuScroll(WMenu *menu)
 {
 	WMenu *smenu;
 	WMenu *omenu = parentMenu(menu);
@@ -1589,13 +1591,14 @@ void wMenuScroll(WMenu * menu, XEvent * event)
 		switch (ev.type) {
 		case EnterNotify:
 			WMHandleEvent(&ev);
+			/* Fall through. */
 		case MotionNotify:
 			x = (ev.type == MotionNotify) ? ev.xmotion.x_root : ev.xcrossing.x_root;
 			y = (ev.type == MotionNotify) ? ev.xmotion.y_root : ev.xcrossing.y_root;
 
 			/* on_border is != 0 if the pointer is between the menu
 			 * and the screen border and is close enough to the border */
-			on_border = isPointNearBoder(menu, x, y);
+			on_border = isPointNearBorder(menu, x, y);
 
 			smenu = wMenuUnderPointer(scr);
 
@@ -1642,6 +1645,7 @@ void wMenuScroll(WMenu * menu, XEvent * event)
 			break;
 		case KeyPress:
 			done = 1;
+			/* Fall through. */
 		default:
 			WMHandleEvent(&ev);
 			break;
@@ -1655,6 +1659,7 @@ void wMenuScroll(WMenu * menu, XEvent * event)
 
 	if (jump_back) {
 		_delay *delayer;
+
 		if (!omenu->jump_back) {
 			delayer = wmalloc(sizeof(_delay));
 			delayer->menu = omenu;
@@ -1664,12 +1669,15 @@ void wMenuScroll(WMenu * menu, XEvent * event)
 			scr->flags.jump_back_pending = 1;
 		} else
 			delayer = omenu->jump_back;
-		WMAddTimerHandler(MENU_JUMP_BACK_DELAY, (WMCallback *) leaving, delayer);
+		WMAddTimerHandler(MENU_JUMP_BACK_DELAY, callback_leaving, delayer);
 	}
 }
 
 static void menuExpose(WObjDescriptor * desc, XEvent * event)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) event;
+
 	wMenuPaint(desc->parent);
 }
 
@@ -1762,7 +1770,7 @@ static void menuMouseDown(WObjDescriptor * desc, XEvent * event)
 		if (!close_on_exit && (bev->state & ControlMask) && smenu && entry->flags.editable) {
 			char buffer[128];
 			char *name;
-			int number = entry_no - 2; /* Entries "New" and "Destroy Last" appear before workspaces */
+			int number = entry_no - 3; /* Entries "New", "Destroy Last" and "Last Used" appear before workspaces */
 
 			name = wstrdup(scr->workspaces[number]->name);
 			snprintf(buffer, sizeof(buffer), _("Type the name for workspace %i:"), number + 1);
@@ -2094,6 +2102,9 @@ static void menuTitleDoubleClick(WCoreWindow * sender, void *data, XEvent * even
 	WMenu *menu = data;
 	int lower;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) sender;
+
 	if (event->xbutton.state & MOD_MASK) {
 		if (menu->flags.lowered) {
 			lower = 0;
@@ -2101,6 +2112,15 @@ static void menuTitleDoubleClick(WCoreWindow * sender, void *data, XEvent * even
 			lower = 1;
 		}
 		changeMenuLevels(menu, lower);
+	} else {
+		if (menu->flags.shaded) {
+			wFrameWindowResize(menu->frame, menu->frame->core->width, menu->frame->top_width +
+					   menu->entry_height*menu->entry_no + menu->frame->bottom_width - 1);
+			menu->flags.shaded = 0;
+		} else {
+			wFrameWindowResize(menu->frame, menu->frame->core->width, menu->frame->top_width - 1);
+			menu->flags.shaded = 1;
+		}
 	}
 }
 
@@ -2114,6 +2134,9 @@ static void menuTitleMouseDown(WCoreWindow * sender, void *data, XEvent * event)
 	int i, lower;
 	Bool started;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) sender;
+
 	/* can't touch the menu copy */
 	if (menu->flags.brother)
 		return;
@@ -2183,7 +2206,7 @@ static void menuTitleMouseDown(WCoreWindow * sender, void *data, XEvent * event)
 						     ButtonMotionMask | ButtonReleaseMask
 						     | ButtonPressMask,
 						     GrabModeAsync, GrabModeAsync, None,
-						     wCursor[WCUR_MOVE], CurrentTime);
+						     wPreferences.cursor[WCUR_MOVE], CurrentTime);
 				}
 			}
 			break;
@@ -2221,6 +2244,10 @@ static void menuCloseClick(WCoreWindow * sender, void *data, XEvent * event)
 	WMenu *parent = menu->parent;
 	int i;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) sender;
+	(void) event;
+
 	if (parent) {
 		for (i = 0; i < parent->cascade_no; i++) {
 			/* find the entry that points to the copy */
@@ -2369,7 +2396,7 @@ static Bool getMenuInfo(WMPropList * info, int *x, int *y, Bool * lowered)
 	return True;
 }
 
-static int restoreMenu(WScreen * scr, WMPropList * menu, int which)
+static int restoreMenu(WScreen *scr, WMPropList *menu)
 {
 	int x, y;
 	Bool lowered = False;
@@ -2381,10 +2408,8 @@ static int restoreMenu(WScreen * scr, WMPropList * menu, int which)
 	if (!getMenuInfo(menu, &x, &y, &lowered))
 		return False;
 
-	if (which & WSS_SWITCHMENU) {
-		OpenSwitchMenu(scr, x, y, False);
-		pmenu = scr->switch_menu;
-	}
+	OpenSwitchMenu(scr, x, y, False);
+	pmenu = scr->switch_menu;
 
 	if (pmenu) {
 		int width = MENUW(pmenu);
@@ -2412,7 +2437,7 @@ static int restoreMenu(WScreen * scr, WMPropList * menu, int which)
 	return False;
 }
 
-static int restoreMenuRecurs(WScreen * scr, WMPropList * menus, WMenu * menu, char *path)
+static int restoreMenuRecurs(WScreen *scr, WMPropList *menus, WMenu *menu, const char *path)
 {
 	WMPropList *key, *entry;
 	char buffer[512];
@@ -2495,7 +2520,7 @@ void wMenuRestoreState(WScreen * scr)
 	skey = WMCreatePLString("SwitchMenu");
 	menu = WMGetFromPLDictionary(menus, skey);
 	WMReleasePropList(skey);
-	restoreMenu(scr, menu, WSS_SWITCHMENU);
+	restoreMenu(scr, menu);
 
 	if (!scr->root_menu) {
 		OpenRootMenu(scr, scr->scr_width * 2, 0, False);
