@@ -28,13 +28,10 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
-#ifdef SHAPE
-#include <X11/extensions/shape.h>
-#endif
 #ifdef KEEP_XKB_LOCK_STATUS
 #include <X11/XKBlib.h>
 #endif				/* KEEP_XKB_LOCK_STATUS */
-#ifdef HAVE_XRANDR
+#ifdef USE_RANDR
 #include <X11/extensions/Xrandr.h>
 #endif
 
@@ -45,7 +42,8 @@
 #include "texture.h"
 #include "pixmap.h"
 #include "menu.h"
-#include "funcs.h"
+#include "window.h"
+#include "main.h"
 #include "actions.h"
 #include "properties.h"
 #include "dock.h"
@@ -55,6 +53,7 @@
 #include "balloon.h"
 #include "geomview.h"
 #include "wmspec.h"
+#include "rootmenu.h"
 
 #include "xinerama.h"
 
@@ -67,24 +66,10 @@
     |SubstructureRedirectMask|ButtonPressMask|ButtonReleaseMask\
     |KeyPressMask|KeyReleaseMask)
 
-/**** Global variables ****/
-extern Cursor wCursor[WCUR_LAST];
-extern WPreferences wPreferences;
-extern Atom _XA_WINDOWMAKER_STATE;
-extern Atom _XA_WINDOWMAKER_NOTICEBOARD;
-
-extern int wScreenCount;
-
-#ifdef KEEP_XKB_LOCK_STATUS
-extern int wXkbSupported;
-#endif
-#ifdef HAVE_XRANDR
-extern int has_randr;
+#ifdef USE_ICCCM_WMREPLACE
+#define REPLACE_WM_TIMEOUT 15
 #endif
 
-extern WDDomain *WDWindowMaker;
-
-/**** Local ****/
 #define STIPPLE_WIDTH 2
 #define STIPPLE_HEIGHT 2
 static char STIPPLE_DATA[] = { 0x02, 0x01 };
@@ -95,6 +80,7 @@ static WMPropList *dApplications = NULL;
 static WMPropList *dWorkspace;
 static WMPropList *dDock;
 static WMPropList *dClip;
+static WMPropList *dDrawers = NULL;
 
 static void make_keys(void)
 {
@@ -105,6 +91,111 @@ static void make_keys(void)
 	dWorkspace = WMCreatePLString("Workspace");
 	dDock = WMCreatePLString("Dock");
 	dClip = WMCreatePLString("Clip");
+	dDrawers = WMCreatePLString("Drawers");
+}
+
+/*
+ * Support for ICCCM 2.0: Window Manager Replacement protocol
+ * See: http://www.x.org/releases/X11R7.6/doc/xorg-docs/specs/ICCCM/icccm.html
+ *
+ * Basically, user should be able to dynamically change its window manager; this is done
+ * cooperatively through a special Selection ("WM_Sn" where 'n' is the X screen number)
+ *
+ * This function does 2 things:
+ *  - it checks if this selection is already owned, which means that another window
+ * manager is running. If it is the case and user specified '--replace' on the command
+ * line, then it asks the WM to shut down;
+ *  - when ok, it sets the selection ownership to ourself, so another window manager
+ * may ask us to shut down (this is handled in "event.c")
+ */
+static Bool replace_existing_wm(WScreen *scr)
+{
+#ifdef USE_ICCCM_WMREPLACE
+	char atomName[16];
+	Window wm;
+	XSetWindowAttributes attribs;
+	XClientMessageEvent event;
+	unsigned long current_time;
+	int ret;
+
+	/* Try to acquire the atom named WM_S<screen> */
+	ret = snprintf(atomName, sizeof(atomName), "WM_S%d", scr->screen);
+	if (ret < 0 || ret == sizeof(atomName)) {
+		werror("out of memory trying to allocate window manager selection atom for screen %d", scr->screen);
+		return False;
+	}
+
+	scr->sn_atom = XInternAtom(dpy, atomName, False);
+	if (! scr->sn_atom)
+		return False;
+
+	/* Check if an existing window manager owns the selection */
+	wm = XGetSelectionOwner(dpy, scr->sn_atom);
+	if (wm) {
+		if (!wPreferences.flags.replace) {
+			wmessage(_("another window manager is running"));
+			wwarning(_("use the --replace flag to replace it"));
+			return False;
+		}
+
+		attribs.event_mask = StructureNotifyMask;
+		if (!XChangeWindowAttributes(dpy, wm, CWEventMask, &attribs))
+			wm = None;
+	}
+#endif
+
+	/* for our window manager info notice board and the selection owner */
+	scr->info_window = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, 10, 10, 0, 0, 0);
+
+#ifdef USE_ICCCM_WMREPLACE
+	/* Try to acquire the selection */
+	current_time = CurrentTime;
+	ret = XSetSelectionOwner(dpy, scr->sn_atom, scr->info_window, current_time);
+	if (ret == BadAtom || ret == BadWindow)
+		return False;
+
+	/* Wait for other window manager to exit */
+	if (wm) {
+		unsigned long wait = 0;
+		unsigned long timeout = REPLACE_WM_TIMEOUT * 1000000L;
+		XEvent event;
+
+		while (wait < timeout) {
+			if (!(wait % 1000000))
+				wmessage(_("waiting %lus for other window manager to exit"), (timeout - wait) / 1000000L);
+
+			if (XCheckWindowEvent(dpy, wm, StructureNotifyMask, &event))
+				if (event.type == DestroyNotify)
+					break;
+
+			wusleep(100000);
+			wait += 100000;
+		}
+
+		if (wait >= timeout) {
+			wwarning(_("other window manager hasn't exited!"));
+			return False;
+		}
+
+		wmessage(_("replacing the other window manager"));
+	}
+
+	if (XGetSelectionOwner(dpy, scr->sn_atom) != scr->info_window)
+		return False;
+
+	event.type = ClientMessage;
+	event.message_type = scr->sn_atom;
+	event.format = 32;
+	event.data.l[0] = (long) current_time;
+	event.data.l[1] = (long) scr->sn_atom;
+	event.data.l[2] = (long) scr->info_window;
+	event.data.l[3] = (long) 0L;
+	event.data.l[4] = (long) 0L;
+	event.window = scr->root_win;
+	XSendEvent(dpy, scr->root_win, False, StructureNotifyMask, (XEvent *) &event);
+#endif
+
+	return True;
 }
 
 /*
@@ -123,6 +214,10 @@ static void make_keys(void)
  */
 static int alreadyRunningError(Display * dpy, XErrorEvent * error)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) dpy;
+	(void) error;
+
 	CantManageScreen = 1;
 	return -1;
 }
@@ -256,7 +351,7 @@ static WPixmap *make3Dots(WScreen * scr)
 
 	XFreeGC(dpy, gc2);
 
-	wpix = wPixmapCreate(scr, pix, mask);
+	wpix = wPixmapCreate(pix, mask);
 	wpix->shared = 1;
 
 	return wpix;
@@ -359,7 +454,6 @@ static void allocGCs(WScreen * scr)
 static void createPixmaps(WScreen * scr)
 {
 	WPixmap *pix;
-	RImage *image;
 
 	/* load pixmaps */
 	pix = wPixmapCreateFromXBMData(scr, (char *)MENU_RADIO_INDICATOR_XBM_DATA,
@@ -402,7 +496,17 @@ static void createPixmaps(WScreen * scr)
 		pix->shared = 1;
 	scr->menu_shade_indicator = pix;
 
-	image = wDefaultGetImage(scr, "Logo", "WMPanel", wPreferences.icon_size);
+	create_logo_image(scr);
+
+	scr->dock_dots = make3Dots(scr);
+
+	/* titlebar button pixmaps */
+	allocButtonPixmaps(scr);
+}
+
+void create_logo_image(WScreen *scr)
+{
+	RImage *image = get_icon_image(scr, "Logo", "WMPanel", 128);
 
 	if (!image) {
 		wwarning(_("could not load logo image for panels: %s"), RMessageForError(RErrorCode));
@@ -410,11 +514,6 @@ static void createPixmaps(WScreen * scr)
 		WMSetApplicationIconImage(scr->wmscreen, image);
 		RReleaseImage(image);
 	}
-
-	scr->dock_dots = make3Dots(scr);
-
-	/* titlebar button pixmaps */
-	allocButtonPixmaps(scr);
 }
 
 /*
@@ -455,24 +554,13 @@ static void createInternalWindows(WScreen * scr)
 	attribs.override_redirect = True;
 	attribs.background_pixmap = None;
 	attribs.background_pixel = scr->white_pixel;
-	attribs.cursor = wCursor[WCUR_DEFAULT];
+	attribs.cursor = wPreferences.cursor[WCUR_NORMAL];
 	vmask |= CWColormap;
 	attribs.colormap = scr->w_colormap;
 	scr->dock_shadow =
 	    XCreateWindow(dpy, scr->root_win, 0, 0, wPreferences.icon_size,
 			  wPreferences.icon_size, 0, scr->w_depth, CopyFromParent, scr->w_visual, vmask, &attribs);
 
-	/* workspace name balloon for clip */
-	vmask = CWBackPixel | CWSaveUnder | CWOverrideRedirect | CWColormap | CWBorderPixel;
-	attribs.save_under = True;
-	attribs.override_redirect = True;
-	attribs.colormap = scr->w_colormap;
-	attribs.background_pixel = scr->icon_back_texture->normal.pixel;
-	attribs.border_pixel = 0;	/* do not care */
-	scr->clip_balloon =
-	    XCreateWindow(dpy, scr->root_win, 0, 0, 10, 10, 0, scr->w_depth,
-			  CopyFromParent, scr->w_visual, vmask, &attribs);
-
 	/* workspace name */
 	vmask = CWBackPixel | CWSaveUnder | CWOverrideRedirect | CWColormap | CWBorderPixel;
 	attribs.save_under = True;
@@ -483,12 +571,6 @@ static void createInternalWindows(WScreen * scr)
 	scr->workspace_name =
 	    XCreateWindow(dpy, scr->root_win, 0, 0, 10, 10, 0, scr->w_depth,
 			  CopyFromParent, scr->w_visual, vmask, &attribs);
-
-	/*
-	 * If the window is clicked without having ButtonPress selected, the
-	 * resulting event will have event.xbutton.window == root.
-	 */
-	XSelectInput(dpy, scr->clip_balloon, ButtonPressMask);
 }
 
 /*
@@ -547,27 +629,29 @@ WScreen *wScreenInit(int screen_number)
 	scr->fakeGroupLeaders = WMCreateArray(16);
 
 	CantManageScreen = 0;
-	oldHandler = XSetErrorHandler((XErrorHandler) alreadyRunningError);
+	oldHandler = XSetErrorHandler(alreadyRunningError);
 
-	event_mask = EVENT_MASK;
-
-	if (wPreferences.disable_root_mouse) {
-		event_mask &= ~(ButtonPressMask | ButtonReleaseMask);
+	/* Do we need to replace an existing window manager? */
+	if (!replace_existing_wm(scr)) {
+		XDestroyWindow(dpy, scr->info_window);
+		wfree(scr);
+		return NULL;
 	}
 
+	event_mask = EVENT_MASK;
 	XSelectInput(dpy, scr->root_win, event_mask);
 
 #ifdef KEEP_XKB_LOCK_STATUS
 	/* Only GroupLock doesn't work correctly in my system since right-alt
 	 * can change mode while holding it too - ]d
 	 */
-	if (wXkbSupported) {
+	if (w_global.xext.xkb.supported) {
 		XkbSelectEvents(dpy, XkbUseCoreKbd, XkbStateNotifyMask, XkbStateNotifyMask);
 	}
 #endif				/* KEEP_XKB_LOCK_STATUS */
 
-#ifdef HAVE_XRANDR
-	if (has_randr)
+#ifdef USE_RANDR
+	if (w_global.xext.randr.supported)
 		XRRSelectInput(dpy, scr->root_win, RRScreenChangeNotifyMask);
 #endif
 
@@ -579,7 +663,7 @@ WScreen *wScreenInit(int screen_number)
 		return NULL;
 	}
 
-	XDefineCursor(dpy, scr->root_win, wCursor[WCUR_ROOT]);
+	XDefineCursor(dpy, scr->root_win, wPreferences.cursor[WCUR_ROOT]);
 
 	/* screen descriptor for raster graphic library */
 	rattr.flags = RC_RenderMode | RC_ColorsPerChannel | RC_StandardColormap;
@@ -590,12 +674,8 @@ WScreen *wScreenInit(int screen_number)
 	if (rattr.colors_per_channel < 2)
 		rattr.colors_per_channel = 2;
 
-	/* will only be accounted for in PseudoColor */
-	if (wPreferences.flags.create_stdcmap) {
-		rattr.standard_colormap_mode = RCreateStdColormap;
-	} else {
-		rattr.standard_colormap_mode = RUseStdColormap;
-	}
+	/* Use standard colormap */
+	rattr.standard_colormap_mode = RUseStdColormap;
 
 	if (getWVisualID(screen_number) >= 0) {
 		rattr.flags |= RC_VisualID;
@@ -612,23 +692,10 @@ WScreen *wScreenInit(int screen_number)
 
 		scr->rcontext = RCreateContext(dpy, screen_number, &rattr);
 	}
-
-	if (!scr->rcontext) {
-		wwarning(_("could not initialize graphics library context: %s"), RMessageForError(RErrorCode));
-		wAbort(False);
-	} else {
-		char **formats;
-		int i = 0;
-
-		formats = RSupportedFileFormats();
-		if (formats) {
-			for (i = 0; formats[i] != NULL; i++) {
-				if (strcmp(formats[i], "TIFF") == 0) {
-					scr->flags.supports_tiff = 1;
-					break;
-				}
-			}
-		}
+	if (scr->rcontext == NULL) {
+		wfatal(_("can't create Context on screen %d, %s"),
+		       screen_number, RMessageForError(RErrorCode));
+		goto abort_no_context;
 	}
 
 	scr->w_win = scr->rcontext->drawable;
@@ -641,6 +708,13 @@ WScreen *wScreenInit(int screen_number)
 
 	if (!scr->wmscreen) {
 		wfatal(_("could not initialize WINGs widget set"));
+		RDestroyContext(scr->rcontext);
+	abort_no_context:
+		WMFreeArray(scr->fakeGroupLeaders);
+		wfree(scr->totalUsableArea);
+		wfree(scr->usableArea);
+		WMFreeBag(scr->stacking_list);
+		wfree(scr);
 		return NULL;
 	}
 
@@ -654,23 +728,22 @@ WScreen *wScreenInit(int screen_number)
 	scr->light_pixel = WMColorPixel(scr->gray);
 	scr->dark_pixel = WMColorPixel(scr->darkGray);
 
-	{
-		XColor xcol;
-		/* frame boder color */
-		wGetColor(scr, FRAME_BORDER_COLOR, &xcol);
-		scr->frame_border_pixel = xcol.pixel;
-	}
-
 	/* create GCs with default values */
 	allocGCs(scr);
 
-	/* for our window manager info notice board. Need to
-	 * create before reading the defaults, because it will be used there.
-	 */
-	scr->info_window = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, 10, 10, 0, 0, 0);
-
 	/* read defaults for this screen */
-	wReadDefaults(scr, WDWindowMaker->dictionary);
+	wReadDefaults(scr, w_global.domain.wmaker->dictionary);
+
+	{
+		XColor xcol;
+		/* frame boder color */
+		wGetColor(scr, WMGetColorRGBDescription(scr->frame_border_color), &xcol);
+		scr->frame_border_pixel = xcol.pixel;
+		wGetColor(scr, WMGetColorRGBDescription(scr->frame_focused_border_color), &xcol);
+		scr->frame_focused_border_pixel = xcol.pixel;
+		wGetColor(scr, WMGetColorRGBDescription(scr->frame_selected_border_color), &xcol);
+		scr->frame_selected_border_pixel = xcol.pixel;
+	}
 
 	createInternalWindows(scr);
 
@@ -695,9 +768,9 @@ WScreen *wScreenInit(int screen_number)
 	PropSetWMakerProtocols(scr->root_win);
 
 	/* setup our noticeboard */
-	XChangeProperty(dpy, scr->info_window, _XA_WINDOWMAKER_NOTICEBOARD,
+	XChangeProperty(dpy, scr->info_window, w_global.atom.wmaker.noticeboard,
 			XA_WINDOW, 32, PropModeReplace, (unsigned char *)&scr->info_window, 1);
-	XChangeProperty(dpy, scr->root_win, _XA_WINDOWMAKER_NOTICEBOARD,
+	XChangeProperty(dpy, scr->root_win, w_global.atom.wmaker.noticeboard,
 			XA_WINDOW, 32, PropModeReplace, (unsigned char *)&scr->info_window, 1);
 
 #ifdef BALLOON_TEXT
@@ -719,7 +792,7 @@ WScreen *wScreenInit(int screen_number)
 	return scr;
 }
 
-void wScreenUpdateUsableArea(WScreen * scr)
+void wScreenUpdateUsableArea(WScreen *scr)
 {
 	/*
 	 * scr->totalUsableArea[] will become the usableArea used for Windowplacement,
@@ -728,23 +801,10 @@ void wScreenUpdateUsableArea(WScreen * scr)
 	 */
 
 	WArea area;
-	int i, dock_head;
-	unsigned long best_area, tmp_area;
-	unsigned int size, position;
-
-	dock_head = scr->xine_info.primary_head;
-	best_area = 0;
-	size = wPreferences.workspace_border_size;
-	position = wPreferences.workspace_border_position;
-
-	if (scr->dock) {
-		WMRect rect;
-		rect.pos.x = scr->dock->x_pos;
-		rect.pos.y = scr->dock->y_pos;
-		rect.size.width = wPreferences.icon_size;
-		rect.size.height = wPreferences.icon_size;
-		dock_head = wGetHeadForRect(scr, rect);
-	}
+	int i;
+	unsigned long tmp_area, best_area = 0;
+	unsigned int size = wPreferences.workspace_border_size;
+	unsigned int position = wPreferences.workspace_border_position;
 
 	for (i = 0; i < wXineramaHeads(scr); ++i) {
 		WMRect rect = wGetRectForHead(scr, i);
@@ -753,16 +813,6 @@ void wScreenUpdateUsableArea(WScreen * scr)
 		scr->totalUsableArea[i].x2 = rect.pos.x + rect.size.width;
 		scr->totalUsableArea[i].y2 = rect.pos.y + rect.size.height;
 
-		if (scr->dock && dock_head == i && (!scr->dock->lowered || wPreferences.no_window_over_dock)) {
-			int offset = wPreferences.icon_size + DOCK_EXTRA_SPACE;
-
-			if (scr->dock->on_right_side) {
-				scr->totalUsableArea[i].x2 -= offset;
-			} else {
-				scr->totalUsableArea[i].x1 += offset;
-			}
-		}
-
 		if (wNETWMGetUsableArea(scr, i, &area)) {
 			scr->totalUsableArea[i].x1 = WMAX(scr->totalUsableArea[i].x1, area.x1);
 			scr->totalUsableArea[i].y1 = WMAX(scr->totalUsableArea[i].y1, area.y1);
@@ -772,23 +822,17 @@ void wScreenUpdateUsableArea(WScreen * scr)
 
 		scr->usableArea[i] = scr->totalUsableArea[i];
 
-#if 0
-		printf("usableArea[%d]: %d %d %d %d\n", i,
-		       scr->usableArea[i].x1, scr->usableArea[i].x2, scr->usableArea[i].y1, scr->usableArea[i].y2);
-#endif
 		if (wPreferences.no_window_over_icons) {
 			if (wPreferences.icon_yard & IY_VERT) {
-				if (wPreferences.icon_yard & IY_RIGHT) {
+				if (wPreferences.icon_yard & IY_RIGHT)
 					scr->totalUsableArea[i].x2 -= wPreferences.icon_size;
-				} else {
+				else
 					scr->totalUsableArea[i].x1 += wPreferences.icon_size;
-				}
 			} else {
-				if (wPreferences.icon_yard & IY_TOP) {
+				if (wPreferences.icon_yard & IY_TOP)
 					scr->totalUsableArea[i].y1 += wPreferences.icon_size;
-				} else {
+				else
 					scr->totalUsableArea[i].y2 -= wPreferences.icon_size;
-				}
 			}
 		}
 
@@ -836,7 +880,7 @@ void wScreenRestoreState(WScreen * scr)
 
 	make_keys();
 
-	if (wScreenCount == 1) {
+	if (w_global.screen_count == 1) {
 		path = wdefaultspathfordomain("WMState");
 	} else {
 		char buf[16];
@@ -845,15 +889,14 @@ void wScreenRestoreState(WScreen * scr)
 	}
 	scr->session_state = WMReadPropListFromFile(path);
 	wfree(path);
-	if (!scr->session_state && wScreenCount > 1) {
+	if (!scr->session_state && w_global.screen_count > 1) {
 		path = wdefaultspathfordomain("WMState");
 		scr->session_state = WMReadPropListFromFile(path);
 		wfree(path);
 	}
 
-	if (!scr->session_state) {
+	if (!scr->session_state)
 		scr->session_state = WMCreatePLDictionary(NULL, NULL);
-	}
 
 	if (!wPreferences.flags.nodock) {
 		state = WMGetFromPLDictionary(scr->session_state, dDock);
@@ -864,9 +907,20 @@ void wScreenRestoreState(WScreen * scr)
 		state = WMGetFromPLDictionary(scr->session_state, dClip);
 		scr->clip_icon = wClipRestoreState(scr, state);
 	}
+	
+	if (!wPreferences.flags.nodrawer) {
+		if (!scr->dock->on_right_side) {
+			/* Drawer tile was created early in wScreenInit() -> wReadDefaults(). At
+			 * that time, scr->dock was NULL and the tile was created as if we were on
+			 * the right side. If we aren't, redo it now. */
+			assert(scr->drawer_tile);
+			RReleaseImage(scr->drawer_tile);
+			scr->drawer_tile = wDrawerMakeTile(scr, scr->icon_tile);
+		}
+		wDrawersRestoreState(scr);
+	}
 
 	wWorkspaceRestoreState(scr);
-
 	wScreenUpdateUsableArea(scr);
 }
 
@@ -897,29 +951,39 @@ void wScreenSaveState(WScreen * scr)
 	if (!wPreferences.flags.nodock) {
 		wDockSaveState(scr, old_state);
 	} else {
-		if ((foo = WMGetFromPLDictionary(old_state, dDock)) != NULL) {
+		foo = WMGetFromPLDictionary(old_state, dDock);
+		if (foo != NULL)
 			WMPutInPLDictionary(scr->session_state, dDock, foo);
-		}
 	}
 	if (!wPreferences.flags.noclip) {
 		wClipSaveState(scr);
 	} else {
-		if ((foo = WMGetFromPLDictionary(old_state, dClip)) != NULL) {
+		foo = WMGetFromPLDictionary(old_state, dClip);
+		if (foo != NULL)
 			WMPutInPLDictionary(scr->session_state, dClip, foo);
-		}
 	}
 
 	wWorkspaceSaveState(scr, old_state);
 
+	if (!wPreferences.flags.nodrawer) {
+		wDrawersSaveState(scr);
+	} else {
+		foo = WMGetFromPLDictionary(old_state, dDrawers);
+		if (foo != NULL)
+			WMPutInPLDictionary(scr->session_state, dDrawers, foo);
+	}
+
+
 	if (wPreferences.save_session_on_exit) {
 		wSessionSaveState(scr);
 	} else {
-		if ((foo = WMGetFromPLDictionary(old_state, dApplications)) != NULL) {
+		foo = WMGetFromPLDictionary(old_state, dApplications);
+		if (foo != NULL)
 			WMPutInPLDictionary(scr->session_state, dApplications, foo);
-		}
-		if ((foo = WMGetFromPLDictionary(old_state, dWorkspace)) != NULL) {
+
+		foo = WMGetFromPLDictionary(old_state, dWorkspace);
+		if (foo != NULL)
 			WMPutInPLDictionary(scr->session_state, dWorkspace, foo);
-		}
 	}
 
 	/* clean up */
@@ -927,7 +991,7 @@ void wScreenSaveState(WScreen * scr)
 
 	wMenuSaveState(scr);
 
-	if (wScreenCount == 1) {
+	if (w_global.screen_count == 1) {
 		str = wdefaultspathfordomain("WMState");
 	} else {
 		char buf[16];
